Imports System.IO
Imports System.ComponentModel
Imports System.Text
Imports iTextSharp
Imports System.Data.OleDb
Imports WESNet
Imports System.Xml
Imports System.Reflection

Namespace WESNet.PDF
    Public MustInherit Class BasePDFList
        Inherits Section

#Region "Private Members"

        'Report Properties
        Private _SQLConnectionString As String
        Private _SQLConnectionObject As SqlClient.SqlConnection
        Private _SQLCommandObject As SqlClient.SqlCommand
        Private _BusinessObjectType As System.Type

        'Properties related to paging of the list
        Private _EnablePaging As Boolean = False
        Private _CurrentPageIndex As Integer = 0
        Private _RecordsPerPage As Integer = 10
        Private _PageCount As Integer
        Private _PagedDataSource As System.Web.UI.WebControls.PagedDataSource
        Private _PagerStyle As PDF.CellStyle

        'Properties related to style of the list

        Private _HeaderStyle As PDF.CellStyle
        Private _ItemStyle As PDF.CellStyle
        Private _AlternatingItemStyle As PDF.CellStyle
        Private _FooterStyle As PDF.CellStyle
        Private _ShowHeader As Boolean = True
        Private _ShowFooter As Boolean = False
        Private _ShowPager As Boolean = True

        'Properties for databinding

        Private _DataKeyField As String
        Private _DataKeys As System.Web.UI.WebControls.DataKeyCollection
        Private _DataKeysArray As ArrayList
        Private _DataMember As String
        Private _DataSource As Object

#End Region

#Region "Public Properties"

        'Enables paging with RecordsPerPage data rows per PDF page
        Public Property EnablePaging() As Boolean
            Get
                Return _EnablePaging
            End Get
            Set(ByVal Value As Boolean)
                _EnablePaging = Value
            End Set
        End Property

        'Gets or sets the current page during databinding
        Public Property CurrentPageIndex() As Integer
            Get
                Return _CurrentPageIndex
            End Get
            Set(ByVal Value As Integer)
                If Value < 0 Then Throw New ArgumentOutOfRangeException("CurrentPageIndex must be >= 0")
                _CurrentPageIndex = Value
            End Set
        End Property

        'Gets or sets the number of data rows per PDF page when EnablePaging is true.
        Public Property RecordsPerPage() As Integer
            Get
                Return _RecordsPerPage
            End Get
            Set(ByVal Value As Integer)
                If Value < 1 Then Throw New ArgumentOutOfRangeException("RecordsPerPage must be > 0")
                _RecordsPerPage = Value
            End Set
        End Property

        'Gets the total number of data pages based on RecordsPerPage data rows per PDF page
        Public ReadOnly Property PageCount() As Integer
            Get
                If _PagedDataSource Is Nothing Then
                    Return _PageCount
                Else
                    Return _PagedDataSource.PageCount
                End If
            End Get
        End Property

        'Provides access to the PagedDataSource object
        Public ReadOnly Property PagedDataSource() As System.Web.UI.WebControls.PagedDataSource
            Get
                If _PagedDataSource Is Nothing Then
                    _PagedDataSource = New System.Web.UI.WebControls.PagedDataSource
                End If
                Return _PagedDataSource
            End Get
        End Property

        'Provides access to the PagerStyle to set font, font-size, etc. of the pager
        Public ReadOnly Property PagerStyle() As CellStyle
            Get
                If _PagerStyle Is Nothing Then
                    _PagerStyle = New CellStyle
                End If
                Return _PagerStyle
            End Get
        End Property

        'Provides access to the HeaderStyle to set font, font-size, etc. of the column headers
        Public ReadOnly Property HeaderStyle() As CellStyle
            Get
                If _HeaderStyle Is Nothing Then
                    _HeaderStyle = New CellStyle
                End If
                Return _HeaderStyle
            End Get
        End Property

        'Provides access to the ItemStyle to set font, font-size, etc. of the data rows.
        Public ReadOnly Property ItemStyle() As CellStyle
            Get
                If _ItemStyle Is Nothing Then
                    _ItemStyle = New CellStyle
                End If
                Return _ItemStyle
            End Get
        End Property

        'Provides access to the AlternatingItemStyle to set font, font-size, etc. of the even
        'numbered data rows.  AlternatingItemStyle inherits any or all of its unset sub-properties
        'from ItemStyle
        Public ReadOnly Property AlternatingItemStyle() As CellStyle
            Get
                If _AlternatingItemStyle Is Nothing Then
                    _AlternatingItemStyle = New CellStyle
                End If
                Return _AlternatingItemStyle
            End Get
        End Property

        'Provides access to the FooterStyle to set font, font-size, etc. of the column footers
        Public ReadOnly Property FooterStyle() As CellStyle
            Get
                If _FooterStyle Is Nothing Then
                    _FooterStyle = New CellStyle
                End If
                Return _FooterStyle
            End Get
        End Property

        'If set to true, show the footer row (Default is false)
        Public Property ShowFooter() As Boolean
            Get
                Return _ShowFooter
            End Get
            Set(ByVal Value As Boolean)
                _ShowFooter = Value
            End Set
        End Property

        'If set to false, do not show the column headers (Default is true)
        Public Property ShowHeader() As Boolean
            Get
                Return _ShowHeader
            End Get
            Set(ByVal Value As Boolean)
                _ShowHeader = Value
            End Set
        End Property

        'If set to false or if EnablePager has been set to false, the Pager row will not be
        'displayed.  To show the Pager row, both EnablePager and ShowPager must be true.
        Public Property ShowPager() As Boolean
            Get
                Return _ShowPager
            End Get
            Set(ByVal Value As Boolean)
                _ShowPager = Value
            End Set
        End Property

        'Optional SQL connection string specifying the connection to the SQL database
        'when the list's data is to be derived from a connection to the database rather than
        'from a bound data source.
        Public Property SQLConnectionString() As String
            Get
                Return _SQLConnectionString
            End Get
            Set(ByVal Value As String)
                _SQLConnectionString = Value
            End Set
        End Property

        'SQLConnection object automatically instantiated when the list's data is being
        'derived from a connection to a database
        Public ReadOnly Property SQLConnectionObject() As SqlClient.SqlConnection
            Get
                If _SQLConnectionObject Is Nothing Then
                    _SQLConnectionObject = New SqlClient.SqlConnection
                End If
                Return _SQLConnectionObject
            End Get
        End Property

        'SQLCommand object automatically instantiated when the list's data is being
        'derived from a connection to a database
        Public ReadOnly Property SQLCommandObject() As SqlClient.SqlCommand
            Get
                If _SQLCommandObject Is Nothing Then
                    _SQLCommandObject = New SqlClient.SqlCommand
                End If
                Return _SQLCommandObject
            End Get
        End Property

        'Class type of the optional business object into which each row of data will be
        'loaded during databinding.  List paging is available only when a BusinessObject class has
        'been created
        Public Property BusinessObjectType() As System.Type
            Get
                Return _BusinessObjectType
            End Get
            Set(ByVal Value As System.Type)
                _BusinessObjectType = Value
            End Set
        End Property

        'CellPadding to be applied to the PDF Table object that will be created
        Public Property CellPadding() As Unit
            Get
                If StyleCreated Then
                    Return CType(Style, TableStyle).CellPadding
                Else
                    Return Unit.Point(2)
                End If
            End Get
            Set(ByVal Value As Unit)
                CType(Style, TableStyle).CellPadding = Value
            End Set
        End Property

        'CellSpacing to be applied to the PDF Table object that will be created
        Public Property CellSpacing() As Unit
            Get
                If StyleCreated Then
                    Return CType(Style, TableStyle).CellSpacing
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                CType(Style, TableStyle).CellSpacing = Value
            End Set
        End Property


        'Selects whether or not Both, Vertical, Horizontal, or None Gridlines will be
        'displayed between columns or rows respectively
        Public Property GridLines() As System.Web.UI.WebControls.GridLines
            Get
                If StyleCreated Then
                    Return CType(Style, TableStyle).GridLines
                Else
                    Return System.Web.UI.WebControls.GridLines.None
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.GridLines)
                CType(Style, TableStyle).GridLines = Value
            End Set
        End Property

        'Properties relating to databinding

        'The DataKey Field should be set to the name of the field whose value will be added to the
        'DataKeys collection during databinding
        Public Property DataKeyField() As String
            Get
                Return _DataKeyField
            End Get
            Set(ByVal Value As String)
                _DataKeyField = Value
            End Set
        End Property

        'A specialized collection of the DataKey values - one for each row
        Public ReadOnly Property DataKeys() As System.Web.UI.WebControls.DataKeyCollection
            Get
                If _DataKeys Is Nothing Then
                    _DataKeys = New System.Web.UI.WebControls.DataKeyCollection(_DataKeysArray)
                End If
                Return _DataKeys
            End Get
        End Property

        'Name of the DataMember (usually a DataSet Table) from which the datasource will be resolved
        Public Property DataMember() As String
            Get
                Return _DataMember
            End Get
            Set(ByVal Value As String)
                _DataMember = Value
            End Set
        End Property

        'The DataSource object from which each row of data will be obtained. Must be IListSource, ICollection,
        'or IEnumerable
        Public Property DataSource() As Object
            Get
                Return _DataSource
            End Get
            Set(ByVal Value As Object)
                If (((Not Value Is Nothing) AndAlso Not TypeOf Value Is IListSource) AndAlso Not TypeOf Value Is IEnumerable) Then
                    Throw New ArgumentException("Invalid DataSource type")
                End If
                _DataSource = Value
            End Set
        End Property
#End Region

#Region "Private/Protected Properties"
        Protected ReadOnly Property DataKeysArray() As ArrayList
            Get
                If _DataKeysArray Is Nothing Then
                    _DataKeysArray = New ArrayList
                End If
                Return _DataKeysArray
            End Get
        End Property
#End Region

#Region "Private/Protected/Friend Methods"


        Protected Overrides Function CreateStyle() As SectionStyle
            Return New TableStyle
        End Function

        Protected Sub ParseDataSource(ByVal Node As XmlNode)
            Dim SQLCommandType As System.Data.CommandType
            Dim SQLCommandText As String
            Dim DataSourceNode As XmlNode = Node.Item("DataSource")
            If Not DataSourceNode Is Nothing Then
                Dim ChildNodes As XmlNodeList = DataSourceNode.ChildNodes
                For Each ChildNode As XmlNode In ChildNodes
                    Select Case ChildNode.Name
                        Case "ConnectionString"
                            Dim Source As String = CType(Common.GetNamedAttribute(ChildNode, "Source", "AppSettings"), String)
                            Select Case Source
                                Case "AppSettings"
                                    Dim AppSettingsKey As String = ChildNode.FirstChild.Value
                                    SQLConnectionString = System.Configuration.ConfigurationSettings.AppSettings(AppSettingsKey)
                                Case "String"
                                    SQLConnectionString = ChildNode.FirstChild.Value
                                Case Else
                            End Select
                            SQLConnectionObject.ConnectionString = SQLConnectionString
                        Case "Command"
                            Dim SQLCommandTypeStr As String = ChildNode.Attributes.GetNamedItem("CommandType").Value()
                            If Not SQLCommandTypeStr Is Nothing Then
                                SQLCommandType = CType([Enum].Parse(GetType(CommandType), SQLCommandTypeStr), CommandType)
                                SQLCommandObject.CommandType = SQLCommandType
                            End If
                            SQLCommandText = ChildNode.FirstChild.InnerText
                            SQLCommandObject.CommandText = SQLCommandText
                        Case "BusinessObject"
                            Dim ObjectType As System.Type
                            Dim BusinessObject As String() = ChildNode.FirstChild.InnerText.Split(","c)
                            If BusinessObject.Length = 2 Then
                                Dim asm As [Assembly] = [Assembly].LoadWithPartialName(BusinessObject(1).Trim())
                                If Not asm Is Nothing Then
                                    ObjectType = asm.GetType(BusinessObject(0).Trim())
                                End If
                            Else
                                ObjectType = Type.GetType(BusinessObject(0).Trim())
                            End If
                            If Not ObjectType Is Nothing Then
                                BusinessObjectType = ObjectType
                            End If
                        Case "Parameters"
                            Dim ParameterNodes As XmlNodeList = ChildNode.ChildNodes
                            For Each ParameterNode As XmlNode In ParameterNodes
                                If ParameterNode.Name <> "Parameter" Then
                                    Throw New Exception("Parameters node must contain one or more Parameter child nodes")
                                Else
                                    Dim ParameterName As String = ParameterNode.Attributes.GetNamedItem("Name").Value
                                    Dim ParameterDataType As String = ParameterNode.Attributes.GetNamedItem("DataType").Value
                                    Dim ParameterValue As String = ParameterNode.Attributes.GetNamedItem("Value").Value

                                    Dim Parameter As New SqlClient.SqlParameter
                                    Parameter.ParameterName = ParameterName
                                    Try
                                        Parameter.DbType = CType([Enum].Parse(GetType(SqlDbType), ParameterDataType), DbType)
                                    Catch ex As Exception
                                        Parameter.DbType = DbType.String
                                    End Try
                                    Parameter.Value = ParameterValue
                                    SQLCommandObject.Parameters.Add(Parameter)
                                End If
                            Next
                    End Select
                Next
            End If
        End Sub

        Protected Sub ParsePDFListStyles(ByVal Node As XmlNode)
            Dim ChildNodes As XmlNodeList = Node.ChildNodes
            For Each ChildNode As XmlNode In ChildNodes
                Select Case ChildNode.Name
                    Case "HeaderStyle", "FooterStyle", "ItemStyle", "AlternatingItemStyle", "PagerStyle"
                        Common.SetSubProperties(Me, ChildNode.Name, ChildNode)
                End Select
            Next
        End Sub

        Protected Function GetSQLData() As Object
            Dim dr As SqlClient.SqlDataReader
            Dim ar As New ArrayList
            Try
                SQLCommandObject.Connection = SQLConnectionObject
                SQLConnectionObject.Open()
                dr = SQLCommandObject.ExecuteReader(CommandBehavior.CloseConnection)
                If BusinessObjectType Is Nothing Then
                    Return dr
                Else
                    Return CBO.FillCollection(dr, BusinessObjectType)
                End If

            Catch ex As Exception
                Throw New Exception("Unable to read from data source " & ex.Message)
            Finally
                If Not BusinessObjectType Is Nothing AndAlso (Not dr Is Nothing AndAlso Not dr.IsClosed) Then
                    dr.Close()
                End If
            End Try
        End Function

        Protected Function GetResolvedDataSource() As IEnumerable
            Dim source1 As IListSource
            Dim list1 As IList
            Dim list2 As ITypedList
            Dim pdc As PropertyDescriptorCollection
            Dim pd As PropertyDescriptor
            Dim obj1, obj2 As Object

            'Any datasource to bind to?
            If DataSource Is Nothing Then Return Nothing

            'Easy case - array, arraylist, etc.
            If TypeOf DataSource Is IEnumerable Then Return CType(DataSource, IEnumerable)

            'Is DataSource (or one of its members an IList?
            If TypeOf DataSource Is IListSource Then
                source1 = CType(DataSource, IListSource)
                list1 = CType(DataSource, IListSource).GetList()
                If Not source1.ContainsListCollection Then
                    Return list1
                Else
                    If Not list1 Is Nothing AndAlso TypeOf list1 Is ITypedList Then
                        list2 = CType(list1, ITypedList)
                        pdc = list2.GetItemProperties(Nothing)
                        If Not pdc Is Nothing AndAlso pdc.Count > 0 Then
                            pd = Nothing
                            If DataMember Is Nothing OrElse DataMember.Length = 0 Then
                                pd = pdc(0)
                            Else
                                pd = pdc.Find(DataMember, True)
                            End If
                            If pd Is Nothing Then
                                Throw New Exception("DataSource does not contain named DataMember (" & DataMember & ")")
                            Else
                                obj1 = list1(0)
                                obj2 = pd.GetValue(obj1)
                                If Not obj2 Is Nothing AndAlso TypeOf obj2 Is IEnumerable Then
                                    Return CType(obj2, IEnumerable)
                                End If
                            End If
                        Else
                            Throw New Exception("DataSource has no DataMembers")
                        End If
                    End If
                End If
                Return Nothing
            End If
        End Function
#End Region

#Region "Public Methods"
        Public Overrides Sub DataBind()
            MyBase.DataBind()
            If DataSource Is Nothing Then
                DataSource = GetSQLData()
            End If
            OnDataBinding(System.EventArgs.Empty)
        End Sub

#End Region

#Region "Constructors"

        Public Sub New()
            'Default section type undefined
        End Sub
        Public Sub New(ByVal SectionType As SectionTypes)
            MyBase.New(SectionType)
            CellPadding = Unit.Point(2.0)
            CellSpacing = Unit.Empty
        End Sub

        Public Sub New(ByVal SectionType As SectionTypes, ByVal Node As XmlNode)
            Me.New(SectionType)
            ParseSectionAttributes(Node)
            ParseSectionStyle(Node)
            ParseDataSource(Node)
            ParsePDFListStyles(Node)
        End Sub
#End Region

#Region "Events"
        Public Event DataBinding As EventHandler
        Public Event ItemCreated As PDFItemEvent
        Public Event ItemDataBound As PDFItemEvent

        Public Delegate Sub PDFItemEvent(ByVal sender As Object, ByVal e As PDFItemEventArgs)
#End Region

#Region "Event Handlers"

        Protected Overridable Sub OnDataBinding(ByVal e As EventArgs)
            RaiseEvent DataBinding(Me, e)
        End Sub

        Protected Overridable Sub OnItemCreated(ByVal e As PDFItemEventArgs)
            RaiseEvent ItemCreated(Me, e)
        End Sub

        Protected Overridable Sub OnItemDataBound(ByVal e As PDFItemEventArgs)
            RaiseEvent ItemDataBound(Me, e)
        End Sub
#End Region

    End Class

    Public Class PDFItemEventArgs
        Private _Item As PDFItem

        Public ReadOnly Property Item() As PDFItem
            Get
                Return _Item
            End Get
        End Property

        Public Sub New(ByVal Item As PDFItem)
            _Item = Item
        End Sub
    End Class

End Namespace
