Imports System.ComponentModel
Imports System.Text
Imports System.Globalization

Namespace WESNet

    'The Border class is used to define borders of cells, tables, pages, and other object as
    'being comprised of four Edge objects - Top, Right, Bottom, and Left.

    'Each Edge object is defined by its Color, Width, and Style

    'Use of the custom type converter BorderConverter, it is possible to specify one or more borders
    'using familiar CSS values such as 'solid 2pt red'

    <TypeConverter(GetType(BorderConverter))> _
    Public Class Border

#Region "Private Members"
        Private _Top As Edge
        Private _Right As Edge
        Private _Bottom As Edge
        Private _Left As Edge
        Private _SetBits As Integer
#End Region

#Region "Shared Members"
        Public Shared ReadOnly Empty As New Border
#End Region

#Region "Private Constants"
        Private Const PROP_BORDERTOP As Integer = &H80
        Private Const PROP_BORDERRIGHT As Integer = &H100
        Private Const PROP_BORDERBOTTOM As Integer = &H200
        Private Const PROP_BORDERLEFT As Integer = &H400
#End Region

#Region "Public Properties"

        Public Property Color() As System.Drawing.Color
            Get
                Return Top.Color
            End Get
            Set(ByVal Value As System.Drawing.Color)
                Top.Color = Value
                Right.Color = Value
                Bottom.Color = Value
                Left.Color = Value
                If Not Value.IsEmpty Then SetBit(PROP_BORDERTOP Or PROP_BORDERRIGHT Or PROP_BORDERBOTTOM Or PROP_BORDERLEFT)
            End Set
        End Property

        Public Property Width() As Unit
            Get
                Return Top.Width
            End Get
            Set(ByVal Value As Unit)
                Top.Width = Value
                Right.Width = Value
                Bottom.Width = Value
                Left.Width = Value
                If Not Value.IsEmpty Then SetBit(PROP_BORDERTOP Or PROP_BORDERRIGHT Or PROP_BORDERBOTTOM Or PROP_BORDERLEFT)
            End Set
        End Property

        Public Property Style() As System.Web.UI.WebControls.BorderStyle
            Get
                Return Top.Style
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.BorderStyle)
                Top.Style = Value
                Right.Style = Value
                Bottom.Style = Value
                Left.Style = Value
                If Value <> Web.UI.WebControls.BorderStyle.NotSet Then SetBit(PROP_BORDERTOP Or PROP_BORDERRIGHT Or PROP_BORDERBOTTOM Or PROP_BORDERLEFT)
            End Set
        End Property

        Public Property Top() As Edge
            Get
                If IsSet(PROP_BORDERTOP) Then
                    Return _Top
                Else
                    If _Top Is Nothing Then
                        _Top = New Edge
                    End If
                    Return _Top
                End If
            End Get
            Set(ByVal Value As Edge)
                _Top = Value
                SetBit(PROP_BORDERTOP, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Right() As Edge
            Get
                If IsSet(PROP_BORDERRIGHT) Then
                    Return _Right
                Else
                    If _Right Is Nothing Then
                        _Right = New Edge
                    End If
                    Return _Right
                End If
            End Get
            Set(ByVal Value As Edge)
                _Right = Value
                SetBit(PROP_BORDERRIGHT, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Bottom() As Edge
            Get
                If IsSet(PROP_BORDERBOTTOM) Then
                    Return _Bottom
                Else
                    If _Bottom Is Nothing Then
                        _Bottom = New Edge
                    End If
                    Return _Bottom
                End If
            End Get
            Set(ByVal Value As Edge)
                _Bottom = Value
                SetBit(PROP_BORDERBOTTOM, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Left() As Edge
            Get
                If IsSet(PROP_BORDERLEFT) Then
                    Return _Left
                Else
                    If _Left Is Nothing Then
                        _Left = New Edge
                    End If
                    Return _Left
                End If
            End Get
            Set(ByVal Value As Edge)
                _Left = Value
                SetBit(PROP_BORDERLEFT, Not Value.IsEmpty)
            End Set
        End Property
#End Region

#Region "Private/Protected Properties"

        Public ReadOnly Property IsEmpty() As Boolean
            Get
                Return _SetBits = 0
            End Get
        End Property

#End Region

#Region "Private/Protected Methods"

        Private Overloads Sub SetBit(ByVal bitMask As Integer)
            _SetBits = _SetBits Or bitMask
        End Sub

        Private Sub ClearBit(ByVal bitMask As Integer)
            _SetBits = _SetBits And (Not bitMask)
        End Sub

        Private Overloads Sub SetBit(ByVal bitMask As Integer, ByVal Condition As Boolean)
            If Condition Then
                SetBit(bitMask)
            Else
                ClearBit(bitMask)
            End If
        End Sub

        Private Function IsSet(ByVal bitMask As Integer) As Boolean
            Return (_SetBits And bitMask) <> 0
        End Function
#End Region

#Region "Public Methods"

        Public Shadows Function Equals(ByVal obj As Object) As Boolean
            If obj Is Nothing Then
                Return False
            Else
                Dim BorderObj As Border = CType(obj, Border)
                Return _Top.Equals(BorderObj.Top) And _Right.Equals(BorderObj.Top) And _Bottom.Equals(BorderObj.Bottom) And _Left.Equals(BorderObj.Left)
            End If
        End Function

        Public Overloads Function ToString() As String
            Return ToString(CultureInfo.CurrentCulture)
        End Function

        Public Overloads Function ToString(ByVal culture As CultureInfo) As String
            If IsEmpty Then Return String.Empty
            Return Top.ToString(culture)
        End Function

        Public Sub CopyFrom(ByVal b As Border)
            If Not b.IsEmpty Then
                With b
                    If .IsSet(PROP_BORDERTOP) And Not .Top.IsEmpty Then
                        Top = New Edge(.Top)
                    End If
                    If .IsSet(PROP_BORDERRIGHT) And Not .Right.IsEmpty Then
                        Right = New Edge(.Right)
                    End If
                    If .IsSet(PROP_BORDERBOTTOM) And Not .Bottom.IsEmpty Then
                        Bottom = New Edge(.Bottom)
                    End If
                    If .IsSet(PROP_BORDERLEFT) And Not .Left.IsEmpty Then
                        Left = New Edge(.Left)
                    End If
                End With
            End If
        End Sub

        Public Sub MergeWith(ByVal b As Border)
            If Not b.IsEmpty Then
                If IsEmpty Then
                    CopyFrom(b)
                Else
                    With b
                        If .IsSet(PROP_BORDERLEFT) And (Not IsSet(PROP_BORDERLEFT) Or Left.IsEmpty) Then
                            Left.MergeWith(.Left)
                            SetBit(PROP_BORDERLEFT)
                        End If
                        If .IsSet(PROP_BORDERRIGHT) And (Not IsSet(PROP_BORDERRIGHT) Or Right.IsEmpty) Then
                            Right.MergeWith(.Right)
                            SetBit(PROP_BORDERRIGHT)
                        End If
                        If .IsSet(PROP_BORDERBOTTOM) And (Not IsSet(PROP_BORDERBOTTOM) Or Bottom.IsEmpty) Then
                            Bottom.MergeWith(.Bottom)
                            SetBit(PROP_BORDERBOTTOM)
                        End If
                        If .IsSet(PROP_BORDERRIGHT) And (Not IsSet(PROP_BORDERRIGHT) Or Right.IsEmpty) Then
                            Right.MergeWith(.Right)
                            SetBit(PROP_BORDERRIGHT)
                        End If
                    End With
                End If
            End If
        End Sub

        Public Overloads Shared Function Parse(ByVal s As String) As Border
            Return Parse(s, CultureInfo.CurrentCulture)
        End Function

        Public Overloads Shared Function Parse(ByVal s As String, ByVal culture As CultureInfo) As Border
            Return New Border(s, culture)
        End Function
#End Region

#Region "Public Constructors"

        Public Sub New()
            Top = New Edge
            Right = New Edge
            Bottom = New Edge
            Left = New Edge
        End Sub

        Public Sub New(ByVal Width As Unit, ByVal Style As Web.UI.WebControls.BorderStyle, ByVal Color As System.Drawing.Color)
            Top = New Edge(Width, Style, Color)
            Right = New Edge(Top)
            Bottom = New Edge(Top)
            Left = New Edge(Top)
        End Sub

        Public Sub New(ByVal s As String)
            Me.New(s, CultureInfo.CurrentCulture)
        End Sub

        Public Sub New(ByVal s As String, ByVal culture As CultureInfo)
            If s Is Nothing OrElse s.Length = 0 Then
                Top = New Edge
                Right = New Edge
                Bottom = New Edge
                Left = New Edge
            Else
                Top = New Edge(s, culture)
                Right = New Edge(Top)
                Bottom = New Edge(Top)
                Left = New Edge(Top)
            End If
        End Sub
#End Region
    End Class
End Namespace