Imports System.Xml
Imports System.Reflection
Imports System.ComponentModel

Namespace WESNet.PDF
    Public Class Common

#Region "Global Constants"
        Public Const MaxPoints As Single = 14400.0!
        Public Const PointsPerCm As Single = PointsPerInch / 2.54!
        Public Const PointsPerMm As Single = PointsPerInch / 2540.0!
        Public Const PointsPerInch As Single = 72.0!
        Public Const PointsPerPica As Single = 12.0!
        Public Const PixelsPerInch As Single = 96.0!
        Public Const PointsPerPixel As Single = PointsPerInch / PixelsPerInch
        Public Const PixelsPerPoint As Single = PixelsPerInch / PointsPerInch
        Public Const MaxPixels As Single = MaxPoints * PixelsPerPoint
        Public Const MaxInches As Single = MaxPoints / PointsPerInch

        'These correspond to the fixed font sizes: XXSmall, XSmall, Small ... , XXLarge
        Public Shared FontSizes As Single() = {6.0!, 8.0!, 10.0!, 12.0!, 14.0!, 16.0!, 18.0!}

#End Region

        'Translate System.Drawing.Color from/to iTextSharp.text.Color spaces
        Public Overloads Shared Function TranslateColor(ByVal Color As System.Drawing.Color) As iTextSharp.text.Color
            If Color.IsEmpty Then
                Return iTextSharp.text.Color.BLACK
            Else
                'Return New iTextSharp.text.Color(Color.R, Color.G, Color.B)
                Return New iTextSharp.text.Color(Color)
            End If
        End Function

        Public Overloads Shared Function TranslateColor(ByVal Color As iTextSharp.text.Color) As System.Drawing.Color
            If Color Is iTextSharp.text.Color.WHITE Then
                Return System.Drawing.Color.Black
            Else
                Return System.Drawing.Color.FromArgb(Color.R, Color.G, Color.B)
            End If
        End Function

        Public Shared Sub SetColor(ByRef DestinationColor As iTextSharp.text.Color, ByVal SourceColor As System.Drawing.Color)
            If Not SourceColor.IsEmpty Then
                DestinationColor = TranslateColor(SourceColor)
            End If
        End Sub

        Public Shared Function CreatePDFFont(ByVal Style As WESNet.PDF.Style) As iTextSharp.text.Font
            Dim Font As iTextSharp.text.Font
            Try
                Dim FontName As String = Style.Font.Name
                Dim FontSize As Single = TranslateFontSize(Style.Font.Size, 12.0!)
                Dim FontStyle As Integer = TranslateFontStyle(Style.Font)
                Dim FontEncoding As String = "" 'iTextSharp.text.FontFactory.DefaultEncoding
                Dim FontEmbedding As Boolean = iTextSharp.text.FontFactory.DefaultEmbedding
                Dim FontColor As iTextSharp.text.Color = TranslateColor(Style.ForeColor)
                Font = iTextSharp.text.FontFactory.GetFont(FontName, FontEncoding, FontEmbedding, FontSize, FontStyle, FontColor)
            Catch ex As Exception
                Font = Nothing 'iTextSharp.text.FontFactory.GetFont(iTextSharp.text.FontFactory.HELVETICA, 12.0!)
            End Try
            Return Font
        End Function

        Public Shared Function TranslateFontStyle(ByVal FontStyle As FontInfo) As Integer
            Dim N As Integer = 0
            If FontStyle.Bold Then N += iTextSharp.text.Font.BOLD
            If FontStyle.Italic Then N += iTextSharp.text.Font.ITALIC
            If FontStyle.Underline Then N += iTextSharp.text.Font.UNDERLINE
            If FontStyle.Strikeout Then N += iTextSharp.text.Font.STRIKETHRU
            Return N
        End Function

        Public Shared Function TranslateFontSize(ByVal FontSize As System.Web.UI.WebControls.FontUnit, ByVal ParentFontSize As Single) As Single
            Dim Value As Single = FontSizes(3)
            Select Case FontSize.Type
                Case Web.UI.WebControls.FontSize.AsUnit
                    Value = UnitToPoints(FontSize.Unit)
                Case Web.UI.WebControls.FontSize.Smaller
                    Value = ParentFontSize - 2.0!
                Case Web.UI.WebControls.FontSize.Larger
                    Value = ParentFontSize + 2.0!
                Case Web.UI.WebControls.FontSize.NotSet
                Case Else
                    Dim i As Integer = CInt(FontSize.Type) - CInt(Web.UI.WebControls.FontSize.XXSmall)
                    If i > 0 And i < 7 Then
                        Value = FontSizes(i)
                    End If
            End Select
            Return Value
        End Function

        Public Shared Sub ApplyFontStyle(ByVal N As Integer, ByVal FontStyle As FontInfo)
            If Not FontStyle Is Nothing Then
                FontStyle.Bold = (N And iTextSharp.text.Font.BOLD) <> 0
                FontStyle.Italic = (N And iTextSharp.text.Font.ITALIC) <> 0
                FontStyle.Underline = (N And iTextSharp.text.Font.UNDERLINE) <> 0
                FontStyle.Strikeout = (N And iTextSharp.text.Font.STRIKETHRU) <> 0
            End If
        End Sub

        Public Shared Function UnitToPoints(ByVal U As System.web.UI.WebControls.Unit) As Single
            If U.Type = Web.UI.WebControls.UnitType.Point Or U.Value = 0.0 Then
                Return CSng(U.Value)
            Else
                Select Case U.Type
                    Case Web.UI.WebControls.UnitType.Cm
                        Return CSng(U.Value * PointsPerCm)
                    Case Web.UI.WebControls.UnitType.Em
                        Throw New NotSupportedException("Cannot convert unit from ems to points")
                    Case Web.UI.WebControls.UnitType.Ex
                        Throw New NotSupportedException("Cannot convert unit from exs to points")
                    Case Web.UI.WebControls.UnitType.Inch
                        Return CSng(U.Value * PointsPerInch)
                    Case Web.UI.WebControls.UnitType.Mm
                        Return CSng(U.Value * PointsPerMm)
                    Case Web.UI.WebControls.UnitType.Percentage
                        Throw New NotSupportedException("Cannot convert unit from percent to points")
                    Case Web.UI.WebControls.UnitType.Pica
                        Return CSng(U.Value * PointsPerPica)
                    Case Web.UI.WebControls.UnitType.Pixel
                        Return CSng(U.Value * PixelsPerPoint)
                End Select
            End If
        End Function

        Public Shared Function TranslateHorizontalAlignment(ByVal Alignment As System.Web.UI.WebControls.HorizontalAlign) As Integer
            Dim N As Integer = iTextSharp.text.Element.ALIGN_UNDEFINED
            Select Case Alignment
                Case Web.UI.WebControls.HorizontalAlign.Center
                    N = iTextSharp.text.Element.ALIGN_CENTER
                Case Web.UI.WebControls.HorizontalAlign.Justify
                    N = iTextSharp.text.Element.ALIGN_JUSTIFIED
                Case Web.UI.WebControls.HorizontalAlign.Left
                    N = iTextSharp.text.Element.ALIGN_LEFT
                Case Web.UI.WebControls.HorizontalAlign.Right
                    N = iTextSharp.text.Element.ALIGN_RIGHT
                Case Web.UI.WebControls.HorizontalAlign.NotSet
                    N = iTextSharp.text.Element.ALIGN_LEFT
            End Select
            Return N
        End Function

        Public Shared Sub SetHorizontalAlignment(ByRef DestinationAlignment As Integer, ByVal SourceAlignment As System.Web.UI.WebControls.HorizontalAlign)
            If SourceAlignment <> Web.UI.WebControls.HorizontalAlign.NotSet Then
                DestinationAlignment = TranslateHorizontalAlignment(SourceAlignment)
            End If
        End Sub

        Public Shared Function TranslateBorders(ByVal B As Border) As Integer
            Dim Borders As Integer = 0
            If Not B.IsEmpty Then
                If B.Top.Style = Web.UI.WebControls.BorderStyle.Solid Then Borders = iTextSharp.text.Cell.TOP_BORDER
                If B.Right.Style = Web.UI.WebControls.BorderStyle.Solid Then Borders = Borders + iTextSharp.text.Cell.RIGHT_BORDER
                If B.Bottom.Style = Web.UI.WebControls.BorderStyle.Solid Then Borders = Borders + iTextSharp.text.Cell.BOTTOM_BORDER
                If B.Left.Style = Web.UI.WebControls.BorderStyle.Solid Then Borders = Borders + iTextSharp.text.Cell.LEFT_BORDER
            End If
            Return Borders
        End Function

        Public Shared Sub SetBorders(ByRef DestinationBorder As Integer, ByVal SourceBorder As Border)
            If Not SourceBorder.IsEmpty Then
                DestinationBorder = TranslateBorders(SourceBorder)
            End If
        End Sub

        Public Shared Function TranslateVerticalAlignment(ByVal Alignment As System.Web.UI.WebControls.VerticalAlign) As Integer
            Dim N As Integer = iTextSharp.text.Element.ALIGN_UNDEFINED
            Select Case Alignment
                Case Web.UI.WebControls.VerticalAlign.Bottom
                    N = iTextSharp.text.Element.ALIGN_BOTTOM
                Case Web.UI.WebControls.VerticalAlign.Middle
                    N = iTextSharp.text.Element.ALIGN_MIDDLE
                Case Web.UI.WebControls.VerticalAlign.Top
                    N = iTextSharp.text.Element.ALIGN_TOP
                Case Web.UI.WebControls.VerticalAlign.NotSet
                    N = iTextSharp.text.Element.ALIGN_MIDDLE
            End Select
            Return N
        End Function

        Public Shared Sub SetVerticalAlignment(ByRef DestinationAlignment As Integer, ByVal SourceAlignment As System.Web.UI.WebControls.VerticalAlign)
            If SourceAlignment <> Web.UI.WebControls.VerticalAlign.NotSet Then
                DestinationAlignment = TranslateVerticalAlignment(SourceAlignment)
            End If
        End Sub

#Region "XML Helper Methods"

        Public Shared Function GetNamedAttribute(ByVal Node As XmlNode, ByVal Name As String, ByVal DefaultValue As Object) As Object
            Dim AttributeNode As XmlNode = Node.Attributes.GetNamedItem(Name)
            If AttributeNode Is Nothing Then
                Return DefaultValue
            Else
                'Dim AttributeType As TypeCode = Type.GetTypeCode(DefaultValue.GetType)
                'Return Convert.ChangeType(AttributeNode.Value, AttributeType)
                Return AttributeNode.Value
            End If
        End Function

        Public Shared Function GetNamedItemValue(ByVal Node As XmlNode, ByVal Name As String, ByVal DefaultValue As Object) As Object
            Dim ItemNode As XmlNode = Node.Item("Name")
            If ItemNode Is Nothing Then
                Return DefaultValue
            Else
                Return ItemNode.Value
            End If
        End Function


        'Set the sub/complex properties of the ParentProperty with the attributes attached to
        'the supplied Node. This is most appropriately used for attributes such as
        'Font-Bold, Font-Size, etc.

        Public Shared Sub SetSubProperties(ByVal Container As Object, ByVal ParentPropertyName As String, ByVal Node As XmlNode)

            Dim Attributes As XmlAttributeCollection = Node.Attributes
            For Each Attribute As XmlAttribute In Attributes
                Dim MethInfo As MethodInfo = Nothing
                Dim FldInfo As FieldInfo = Nothing
                Dim PropInfo As PropertyInfo = Nothing
                Dim FullPropertyName As String = (ParentPropertyName & "." & Attribute.Name.Replace("-"c, "."c)).TrimStart(New Char() {"."c})
                Dim obj As Object = Container
                Dim objType As Type = obj.GetType
                Dim idx1 As Integer = 0
                Dim names As String() = FullPropertyName.Split("."c)
                Dim name As String
                Do While idx1 <= names.Length - 1
                    name = names(idx1)
                    Dim MbrInfo As MemberInfo() = objType.GetMember(name) ', BindingFlags.GetProperty Or BindingFlags.GetField)
                    If MbrInfo Is Nothing OrElse MbrInfo.Length = 0 Then
                        objType = Nothing
                        Exit Do
                    End If
                    If MbrInfo(0).MemberType = MemberTypes.Property Then
                        PropInfo = CType(MbrInfo(0), PropertyInfo)
                        objType = PropInfo.PropertyType
                        If idx1 < names.Length - 1 Then
                            MethInfo = PropInfo.GetGetMethod
                            obj = MethInfo.Invoke(obj, Nothing)
                        Else
                            MethInfo = PropInfo.GetSetMethod
                        End If
                    ElseIf MbrInfo(0).MemberType = MemberTypes.Field Then
                        FldInfo = CType(MbrInfo(0), FieldInfo)
                        objType = FldInfo.FieldType
                        obj = FldInfo.GetValue(obj)
                    End If
                    idx1 += 1
                Loop
                If Not objType Is Nothing Then
                    Dim SubPropertyValue As Object = Attribute.Value
                    Dim SubPropertyConverter As TypeConverter = TypeDescriptor.GetConverter(objType)
                    Dim SubPropertyValueObject As Object = SubPropertyConverter.ConvertFrom(SubPropertyValue)
                    If Not MethInfo Is Nothing Then
                        Dim objArray2 As Object() = New Object() {SubPropertyValueObject}
                        Dim objArray1 As Object() = objArray2
                        MethInfo.Invoke(obj, objArray1)
                    Else
                        If Not FldInfo Is Nothing Then
                            FldInfo.SetValue(obj, SubPropertyValue)
                        End If
                    End If
                End If
            Next
        End Sub
#End Region

    End Class
End Namespace
