Imports System.Globalization
Imports System.Text
Imports System.ComponentModel

Namespace WESNet

    'The Edge class defines the properties of an object's edge
    '  Width - Unit width such as 2px, 5pt, 0.5in
    '  Style - Style of edge such as solid, dashed
    '  Color - System.Color of edge such as red, #FF0036

    <TypeConverter(GetType(EdgeConverter))> _
    Public Class Edge

#Region "Private Members"
        Private _Width As Unit
        Private _Style As System.Web.UI.WebControls.BorderStyle
        Private _Color As System.Drawing.Color
        Private _SetBits As Integer
#End Region

#Region "Constants"
        Private Const PROP_BORDERWIDTH As Integer = &H10
        Private Const PROP_BORDERSTYLE As Integer = &H20
        Private Const PROP_BORDERCOLOR As Integer = &H40
#End Region

#Region "Public Shared Members"
        Public Shared ReadOnly Empty As New Edge
#End Region

#Region "Public Properties"
        Public Property Width() As Unit
            Get
                If IsSet(PROP_BORDERWIDTH) Then
                    Return _Width
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                If Value.ToPoints < 0.0 Then
                    Throw New ArgumentOutOfRangeException("Border width must be >= 0")
                End If
                _Width = Value
                SetBit(PROP_BORDERWIDTH, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Style() As System.Web.UI.WebControls.BorderStyle
            Get
                If IsSet(PROP_BORDERSTYLE) Then
                    Return _Style
                Else
                    Return Web.UI.WebControls.BorderStyle.NotSet
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.BorderStyle)
                If Value < Web.UI.WebControls.BorderStyle.NotSet Or Value > Web.UI.WebControls.BorderStyle.Outset Then
                    Throw New ArgumentOutOfRangeException("Border style value is invalid")
                End If
                _Style = Value
                SetBit(PROP_BORDERSTYLE, Value <> Web.UI.WebControls.BorderStyle.NotSet)
            End Set
        End Property

        Public Property Color() As System.Drawing.Color
            Get
                If IsSet(PROP_BORDERCOLOR) Then
                    Return _Color
                Else
                    Return System.Drawing.Color.Empty
                End If
            End Get
            Set(ByVal Value As System.Drawing.Color)
                _Color = Value
                SetBit(PROP_BORDERCOLOR, Not Value.IsEmpty)
            End Set
        End Property
#End Region

#Region "Private/Protected Properties"

        Public ReadOnly Property IsEmpty() As Boolean
            Get
                Return _SetBits = 0
            End Get
        End Property

#End Region

#Region "Private/Protected Methods"

        Private Overloads Sub SetBit(ByVal bitMask As Integer)
            _SetBits = _SetBits Or bitMask
        End Sub

        Private Sub ClearBit(ByVal bitMask As Integer)
            _SetBits = _SetBits And (Not bitMask)
        End Sub

        Private Overloads Sub SetBit(ByVal bitMask As Integer, ByVal Condition As Boolean)
            If Condition Then
                SetBit(bitMask)
            Else
                ClearBit(bitMask)
            End If
        End Sub

        Private Function IsSet(ByVal bitMask As Integer) As Boolean
            Return (_SetBits And bitMask) <> 0
        End Function

        Private Function IsUnit(ByVal s As String) As Boolean
            Dim rgx As New RegularExpressions.Regex("^(\d+[\.\,]{0,1}\d*)\s*(cm|em|ex|in|pc|pt|px|%){0,1}", RegularExpressions.RegexOptions.IgnoreCase)
            Return rgx.Match(s).Success
        End Function

        Private Function IsBorderStyle(ByVal s As String) As Boolean
            Dim rgx As New RegularExpressions.Regex("(notset|none|solid|dotted|dashed|double|groove|ridge|inset|outset|inherit)", RegularExpressions.RegexOptions.IgnoreCase)
            Return rgx.Match(s).Success
        End Function

        Private Function IsColor(ByVal s As String) As Boolean
            Dim wcc As New System.Web.UI.WebControls.WebColorConverter
            Return wcc.IsValid(s)
        End Function
#End Region

#Region "Public Methods"

        Public Shadows Function Equals(ByVal obj As Object) As Boolean
            If obj Is Nothing Then
                Return False
            Else
                Dim BorderObj As Edge = CType(obj, Edge)
                Return _Width.Equals(BorderObj.Width) And _Style = BorderObj.Style And Color.Equals(BorderObj.Color)
            End If
        End Function

        Public Overloads Function ToString(ByVal culture As CultureInfo) As String
            If IsEmpty Then Return String.Empty
            Return Width.ToString(culture) & " " & Style.ToString() & Color.ToString()
        End Function

        Public Sub CopyFrom(ByVal b As Edge)
            If Not b.IsEmpty Then
                With b
                    If .IsSet(PROP_BORDERWIDTH) And Not .Width.IsEmpty Then
                        Width = .Width
                    End If
                    If .IsSet(PROP_BORDERSTYLE) And Not .Style = Web.UI.WebControls.BorderStyle.NotSet Then
                        Style = .Style
                    End If
                    If .IsSet(PROP_BORDERCOLOR) And Not .Color.IsEmpty Then
                        Color = .Color
                    End If
                End With
            End If
        End Sub

        Public Sub MergeWith(ByVal b As Edge)
            If Not b.IsEmpty Then
                If IsEmpty Then
                    CopyFrom(b)
                Else
                    With b
                        If .IsSet(PROP_BORDERWIDTH) And (Not IsSet(PROP_BORDERWIDTH) Or Width.IsEmpty) Then
                            Width = .Width
                        End If
                        If .IsSet(PROP_BORDERSTYLE) And (Not IsSet(PROP_BORDERSTYLE) Or Style = Web.UI.WebControls.BorderStyle.NotSet) Then
                            Style = .Style
                        End If
                        If .IsSet(PROP_BORDERCOLOR) And (Not IsSet(PROP_BORDERCOLOR) Or Color.IsEmpty) Then
                            Color = .Color
                        End If
                    End With
                End If
            End If
        End Sub

        Public Overloads Shared Function Parse(ByVal s As String) As Edge
            Return Parse(s, CultureInfo.CurrentCulture)
        End Function

        Public Overloads Shared Function Parse(ByVal s As String, ByVal culture As CultureInfo) As Edge
            Return New Edge(s, culture)
        End Function
#End Region

#Region "Public Constructors"

        Public Sub New()
            _Width = Unit.Empty
            _Style = Web.UI.WebControls.BorderStyle.NotSet
            _Color = System.Drawing.Color.Empty
        End Sub

        Public Sub New(ByVal E As Edge)
            Width = E.Width
            Style = E.Style
            Color = E.Color
        End Sub

        Public Sub New(ByVal W As Unit, ByVal S As Web.UI.WebControls.BorderStyle, ByVal C As System.Drawing.Color)
            Width = W
            Style = S
            Color = C
        End Sub

        Public Sub New(ByVal s As String)
            Me.New(s, CultureInfo.CurrentCulture)
        End Sub

        Public Sub New(ByVal s As String, ByVal culture As CultureInfo)
            If s Is Nothing OrElse s.Length = 0 Then
                _Width = Unit.Empty
                _Style = Web.UI.WebControls.BorderStyle.NotSet
                _Color = System.Drawing.Color.Empty
                Exit Sub
            End If
            Dim properties As String() = s.Split(" "c)
            For i As Integer = 0 To properties.Length - 1
                If Not IsSet(PROP_BORDERWIDTH) And IsUnit(properties(i)) Then
                    Width = Unit.Parse(properties(i))
                ElseIf Not IsSet(PROP_BORDERSTYLE) And IsBorderStyle(properties(i)) Then
                    Style = CType([Enum].Parse(GetType(System.Web.UI.WebControls.BorderStyle), properties(i), True), System.Web.UI.WebControls.BorderStyle)
                ElseIf Not IsSet(PROP_BORDERCOLOR) And IsColor(properties(i)) Then
                    Dim wcc As New System.Web.UI.WebControls.WebColorConverter
                    Color = CType(wcc.ConvertFrom(properties(i)), System.Drawing.Color)
                End If
            Next
            If Not IsSet(PROP_BORDERSTYLE) OrElse Style = Web.UI.WebControls.BorderStyle.None Then
                If IsSet(PROP_BORDERWIDTH) Then
                    Width = Unit.Pixel(0.0!)
                End If
            End If
        End Sub
#End Region

    End Class
End Namespace
