Imports System.ComponentModel

Namespace WESNet.PDF

    'Because the System.Web.FontInfo class can neither be instantiated or inherited,
    'we must define our own substitue. It's persistence will be managed by its owner class
    'of the type PDF.Style

    <TypeConverter(GetType(ExpandableObjectConverter))> _
    Public Class FontInfo

#Region "Private Members"
        Private _owner As PDF.Style
#End Region

#Region "Private Constants"
        Private Const PROP_FONT_BOLD As Integer = &H8
        Private Const PROP_FONT_ITALIC As Integer = &H10
        Private Const PROP_FONT_NAMES As Integer = &H20
        Private Const PROP_FONT_OVERLINE As Integer = &H40
        Private Const PROP_FONT_SIZE As Integer = &H80
        Private Const PROP_FONT_STRIKEOUT As Integer = &H100
        Private Const PROP_FONT_UNDERLINE As Integer = &H200
#End Region

#Region "Public Properties"

        Public Property Bold() As Boolean
            Get
                If _owner.IsSet(PROP_FONT_BOLD) Then
                    Return _owner._FontBold
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                _owner._FontBold = Value
                _owner.SetBit(PROP_FONT_BOLD)
            End Set
        End Property

        Public Property Italic() As Boolean
            Get
                If _owner.IsSet(PROP_FONT_ITALIC) Then
                    Return _owner._FontItalic
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                _owner._FontItalic = Value
                _owner.SetBit(PROP_FONT_ITALIC)
            End Set
        End Property

        Public Property Name() As String
            Get
                Dim FontNames As String() = Names
                If FontNames.Length > 0 Then
                    Return FontNames(0)
                Else
                    Return String.Empty
                End If
            End Get
            Set(ByVal Value As String)
                If Value Is Nothing Then
                    Throw New ArgumentNullException("Font name cannot be nothing")
                Else
                    If Value.Length = 0 Then
                        Names = Nothing
                    Else
                        Names = New String() {Value}
                    End If
                End If
            End Set
        End Property

        Public Property Names() As String()
            Get
                If _owner.IsSet(PROP_FONT_NAMES) Then
                    Dim FontNames As String() = _owner._FontNames
                    If Not FontNames Is Nothing Then
                        Return FontNames
                    End If
                End If
                Return New String() {String.Empty}
            End Get
            Set(ByVal Value As String())
                _owner._FontNames = Value
                _owner.SetBit(PROP_FONT_NAMES)
            End Set
        End Property

        Public Property Overline() As Boolean
            Get
                If _owner.IsSet(PROP_FONT_OVERLINE) Then
                    Return _owner._FontOverline
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                _owner._FontOverline = Value
                _owner.SetBit(PROP_FONT_OVERLINE)
            End Set
        End Property

        '<TypeConverter(GetType(System.Web.UI.WebControls.FontUnitConverter))> _
        Public Property Size() As System.Web.UI.WebControls.FontUnit
            Get
                If _owner.IsSet(PROP_FONT_SIZE) Then
                    Return _owner._FontSize
                Else
                    Return System.Web.UI.WebControls.FontUnit.Empty
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.FontUnit)
                If Value.Type = Web.UI.WebControls.FontSize.AsUnit AndAlso Value.Unit.Value < 0 Then
                    Throw New ArgumentOutOfRangeException("Invalid font size value")
                Else
                    _owner._FontSize = Value
                    _owner.SetBit(PROP_FONT_SIZE)
                End If
            End Set
        End Property

        Public Property Strikeout() As Boolean
            Get
                If _owner.IsSet(PROP_FONT_STRIKEOUT) Then
                    Return _owner._FontStrikeout
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                _owner._FontStrikeout = Value
                _owner.SetBit(PROP_FONT_STRIKEOUT)
            End Set
        End Property

        Public Property Underline() As Boolean
            Get
                If _owner.IsSet(PROP_FONT_UNDERLINE) Then
                    Return _owner._FontUnderline
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                _owner._FontUnderline = Value
                _owner.SetBit(PROP_FONT_UNDERLINE)
            End Set
        End Property
#End Region

#Region "Private/Friend Properties"
        Friend ReadOnly Property Owner() As PDF.Style
            Get
                Return _owner
            End Get
        End Property
#End Region

#Region "Constructor"
        Friend Sub New(ByVal owner As PDF.Style)
            _owner = owner
        End Sub
#End Region

#Region "Public Methods"
        Public Sub CopyFrom(ByVal f As FontInfo)
            If Not f Is Nothing Then
                With f.Owner
                    If .IsSet(PROP_FONT_BOLD) AndAlso f.Bold Then
                        Bold = True
                    End If
                    If .IsSet(PROP_FONT_ITALIC) AndAlso f.Italic Then
                        Italic = True
                    End If
                    If .IsSet(PROP_FONT_NAMES) Then
                        Names = f.Names
                    End If
                    If .IsSet(PROP_FONT_OVERLINE) AndAlso f.Overline Then
                        Overline = True
                    End If
                    If .IsSet(PROP_FONT_SIZE) AndAlso Not f.Size.IsEmpty Then
                        Size = f.Size
                    End If
                    If .IsSet(PROP_FONT_STRIKEOUT) AndAlso f.Strikeout Then
                        Strikeout = f.Strikeout
                    End If
                    If .IsSet(PROP_FONT_UNDERLINE) AndAlso f.Underline Then
                        Underline = f.Underline
                    End If
                End With
            End If
        End Sub

        Public Sub MergeWith(ByVal f As FontInfo)
            If Not f Is Nothing Then
                With f.Owner
                    If .IsSet(PROP_FONT_BOLD) And Not _owner.IsSet(PROP_FONT_BOLD) Then
                        Bold = f.Bold
                    End If
                    If .IsSet(PROP_FONT_ITALIC) And Not _owner.IsSet(PROP_FONT_ITALIC) Then
                        Italic = f.Italic
                    End If
                    If .IsSet(PROP_FONT_NAMES) And Not _owner.IsSet(PROP_FONT_NAMES) Then
                        Names = f.Names
                    End If
                    If .IsSet(PROP_FONT_OVERLINE) And Not _owner.IsSet(PROP_FONT_OVERLINE) Then
                        Overline = f.Overline
                    End If
                    If .IsSet(PROP_FONT_SIZE) And (Not _owner.IsSet(PROP_FONT_SIZE) Or Size.IsEmpty) Then
                        Size = f.Size
                    End If
                    If .IsSet(PROP_FONT_STRIKEOUT) And Not _owner.IsSet(PROP_FONT_STRIKEOUT) Then
                        Strikeout = f.Strikeout
                    End If
                    If .IsSet(PROP_FONT_UNDERLINE) And Not _owner.IsSet(PROP_FONT_UNDERLINE) Then
                        Underline = f.Underline
                    End If
                End With
            End If
        End Sub
#End Region

    End Class
End Namespace
