Imports System.Xml
Imports System.Text.RegularExpressions

Namespace WESNet.PDF
    Public Class Label
        Inherits Section

#Region "Private Members"
        Private _Text As String
        Private _IgnoreWhiteSpace As Boolean = True
#End Region

#Region "Public Properties"

        'The text of the label section that will be displayed in the PDF report.
        'The following tokens when found in the text will be replaced with the appropriate
        'values:

        '   [Author], [Creator], [DateCreated], [Subject], [Title], [Today], [Time]
        '   [CurrentPage] is not working at this time due to limitations of iTextSharp

        Public Property Text() As String
            Get
                Return _Text
            End Get
            Set(ByVal Value As String)
                _Text = Value
            End Set
        End Property

        'If true, ignores whitespace at beginning/end of lines when parsing XML report definition
        Public Property IgnoreWhiteSpace() As Boolean
            Get
                Return _IgnoreWhiteSpace
            End Get
            Set(ByVal Value As Boolean)
                _IgnoreWhiteSpace = Value
            End Set
        End Property

#End Region

#Region "Private/Protected Methods"

        Private Sub ParseLabel(ByVal Node As XmlNode)
            Dim TextNode As XmlNode = Node.Item("Text")
            If TextNode Is Nothing Then
                Visible = False
            Else
                Dim tmpTxt As String = TextNode.InnerText.Trim(New Char() {" "c})
                Try
                    IgnoreWhiteSpace = CType(Common.GetNamedAttribute(TextNode, "IgnoreWhiteSpace", True), Boolean)
                    If IgnoreWhiteSpace Then
                        Text = Regex.Replace(tmpTxt, "\s+", " ")
                    Else
                        Text = tmpTxt
                    End If
                Catch ex As Exception
                    Throw New XmlException("IgnoreWhiteSpace attribute of text node must be 'True' or 'False'")
                End Try
                Dim StyleNode As XmlNode = Node.Item("Style")
                If Not StyleNode Is Nothing Then
                    Common.SetSubProperties(Style, "", StyleNode)
                End If
            End If
        End Sub

        Private Function ReplaceTokens(ByVal Text As String, ByVal writer As PDFWriter) As String
            Dim t As String = Text
            't = t.Replace("[CurrentPage]", writer.PDFWriter.PageNumber.ToString)
            t = t.Replace("[Author]", writer.Author)
            t = t.Replace("[Creator]", writer.Creator)
            t = t.Replace("[DateCreated]", writer.DateCreated.ToShortDateString)
            t = t.Replace("[Subject]", writer.Subject)
            t = t.Replace("[Title]", writer.Title)
            t = t.Replace("[Today]", DateTime.Today.ToShortDateString)
            t = t.Replace("[Time]", DateTime.Now.ToShortDateString)
            Return t
        End Function

#End Region

#Region "Public Methods"

        Public Overrides Function RenderedContent(ByVal writer As PDFWriter) As iTextSharp.text.IElement
            If (Not Text Is Nothing AndAlso Text.Length > 0) Then
                'Text = ReplaceTokens(Text, writer)
                'Dim Font As iTextSharp.text.Font = Common.CreatePDFFont(Style)
                'Dim Phrase As New iTextSharp.text.Phrase
                'Dim TokenName As String = "[TotalPages]"
                'Dim P1 As Integer = Text.IndexOf(TokenName)
                'If P1 <= 0 Then
                '    Phrase.Add(New iTextSharp.text.Chunk(Text, Font))
                'Else
                '    Dim P2 As Integer = P1 + TokenName.Length - 1
                '    Dim L As Integer = Text.Length
                '    Phrase.Add(New iTextSharp.text.Chunk(Text.Substring(0, P1), Font))
                '    Dim Chunk As New iTextSharp.text.Chunk("     ", Font)
                '    Dim TokenObject As Token
                '    TokenName = TokenName.Trim(New Char() {"["c, "]"c})
                '    If writer.Tokens(TokenName) Is Nothing Then
                '        TokenObject = New Token(TokenName, GetType(Integer)).AddInstance(Font, "{0:###0}")
                '        writer.Tokens(TokenName) = TokenObject
                '    Else
                '        TokenObject = CType(writer.Tokens(TokenName), Token)
                '        TokenObject.AddInstance(Font, "{0:###0}")
                '    End If
                '    Chunk.SetGenericTag(TokenObject.LastInstance.Name)
                '    Phrase.Add(Chunk)
                '    If P2 < L - 1 Then
                '        Phrase.Add(New iTextSharp.text.Chunk(Text.Substring(P2 + 1, L - 1 - P2), Font))
                '    End If
                'End If
                'Return Phrase
                Return ParseLabelText(writer)
            End If
        End Function

        Public Function ParseLabelText(ByVal writer As PDFWriter) As iTextSharp.text.Phrase
            Dim rgx As New Regex("([^[]*)(\[[a-zA-Z0-9]*\])*")
            Dim m As Match = rgx.Match(Text)
            Dim g As Group
            Dim Phrase As New iTextSharp.text.Phrase
            Dim Font As iTextSharp.text.Font = Common.CreatePDFFont(Style)
            Dim Chunk As iTextSharp.text.Chunk
            Dim TokenName As String
            Dim TokenValue As String
            While m.Success
                If Not m.Groups(1) Is Nothing AndAlso m.Groups(1).Length > 0 Then
                    Chunk = New iTextSharp.text.Chunk(m.Groups(1).Value, Font)
                    Phrase.Add(Chunk)
                End If
                If Not m.Groups(2) Is Nothing AndAlso m.Groups(2).Length > 0 Then
                    TokenName = m.Groups(2).Value
                    TokenValue = Nothing
                    Select Case TokenName
                        Case "[Author]"
                            TokenValue = writer.Author
                        Case "[Creator]"
                            TokenValue = writer.Creator
                        Case "[DateCreated]"
                            TokenValue = writer.DateCreated.ToShortDateString
                        Case "[Subject]"
                            TokenValue = writer.Subject
                        Case "[Title]"
                            TokenValue = writer.Title
                        Case "[Today]"
                            TokenValue = DateTime.Today.ToShortDateString
                        Case "[Time]"
                            TokenValue = DateTime.Now.ToShortTimeString
                        Case "[CurrentPage]", "[TotalPages]"
                            TokenValue = "      "
                        Case Else
                            Throw New iTextSharp.text.DocumentException("Invalid token " & TokenName & " found in text")
                    End Select
                    Chunk = New iTextSharp.text.Chunk(TokenValue, Font)
                    If TokenName = "[CurrentPage]" Or TokenName = "[TotalPages]" Then
                        TokenName = TokenName.Trim(New Char() {"["c, "]"c})
                        Dim TokenObject As Token
                        If writer.Tokens(TokenName) Is Nothing Then
                            TokenObject = New Token(TokenName, GetType(Integer)).AddInstance(Font, "{0:###0}")
                            writer.Tokens(TokenName) = TokenObject
                        Else
                            TokenObject = CType(writer.Tokens(TokenName), Token)
                            TokenObject.AddInstance(Font, "{0:###0}")
                        End If
                        Chunk.SetGenericTag(TokenObject.LastInstance.Name)
                    End If
                    Phrase.Add(Chunk)

                End If
                m = m.NextMatch()
            End While
            Return Phrase
        End Function

#End Region

#Region "Constructors"
        Public Sub New()
            MyBase.New(SectionTypes.Label)
        End Sub

        Protected Sub New(ByVal SectionType As SectionTypes)
            MyBase.New(SectionType)
        End Sub

        Public Sub New(ByVal Text As String)
            Me.New()
            Me.Text = Text
        End Sub

        Protected Sub New(ByVal SectionType As SectionTypes, ByVal Text As String)
            MyBase.New(SectionType)
            Me.Text = Text
        End Sub

        Public Sub New(ByVal Node As XmlNode)
            MyBase.New(SectionTypes.Label, Node)
            ParseLabel(Node)
        End Sub

        Protected Sub New(ByVal SectionType As SectionTypes, ByVal Node As XmlNode)
            MyBase.New(SectionType)
            ParseLabel(Node)
        End Sub
#End Region
    End Class
End Namespace
