Imports System.IO
Imports iTextSharp
Imports System.Xml
Imports System.Reflection
Imports System.ComponentModel
Imports System.Text.RegularExpressions

Namespace WESNet.PDF
    Public Class PDFGenerator

#Region "Private Members"

        'PDF Document and page properties
        Private _PDFWriter As PDFWriter
        Private _Processing As ProcessingMethods = ProcessingMethods.Automatic
        Private _Destination As Destinations = Destinations.HttpResponse
        Private _OpenNewWindow As Boolean = True
        Private _OutputFilePath As String
        Private _Title As String
        Private _TitleStyle As CellStyle
        Private _TitleVisible As Boolean = True
        Private _Author As String
        Private _AuthorVisible As Boolean = True
        Private _Subject As String
        Private _SubjectVisible As Boolean = True
        Private _DateCreated As DateTime = Now()
        Private _DateCreatedVisible As Boolean = True
        Private _IsRendered As Boolean = False
        Private _Sections As SectionCollection

        Private _ReportPath As String
#End Region


#Region "Public Properties"

        'Complete path to the XML report definition file
        Public Property ReportPath() As String
            Get
                Return _ReportPath
            End Get
            Set(ByVal Value As String)
                _ReportPath = Value
            End Set
        End Property

        'Provides access to the PDFWriter class object
        Public ReadOnly Property PDFWriter() As PDFWriter
            Get
                If _PDFWriter Is Nothing Then
                    _PDFWriter = New PDFWriter
                End If
                Return _PDFWriter
            End Get
        End Property

        'Processing methods - Manual, ManualDataBind, ManualOutput, Automatic
        Public Property Processing() As ProcessingMethods
            Get
                Return _Processing
            End Get
            Set(ByVal Value As ProcessingMethods)
                _Processing = Value
            End Set
        End Property

        'Destinations - HttpResponse, and File
        Public Property Destination() As Destinations
            Get
                Return _Destination
            End Get
            Set(ByVal Value As Destinations)
                _Destination = Value
            End Set
        End Property

        'If set true, PDF output on the response stream opens in new window
        Public Property OpenNewWindow() As Boolean
            Get
                Return _OpenNewWindow
            End Get
            Set(ByVal Value As Boolean)
                _OpenNewWindow = Value
            End Set
        End Property

        'Compete path of the PDF output file that will be written
        Public Property OutputFilePath() As String
            Get
                Return _OutputFilePath
            End Get
            Set(ByVal Value As String)
                _OutputFilePath = Value
            End Set
        End Property

        'When used with ASP.NET web page, returns the current HttpResponse object
        Public ReadOnly Property Response() As System.Web.HttpResponse
            Get
                If System.Web.HttpContext.Current Is Nothing Then
                    Return Nothing
                Else
                    Return System.Web.HttpContext.Current.Response
                End If
            End Get
        End Property

        'Report Title
        Public Property Title() As String
            Get
                Return _Title
            End Get
            Set(ByVal Value As String)
                _Title = Value
            End Set
        End Property

        'Style of Report Title
        Public ReadOnly Property TitleStyle() As PDF.CellStyle
            Get
                If _TitleStyle Is Nothing Then
                    _TitleStyle = New CellStyle
                End If
                Return _TitleStyle
            End Get
        End Property

        'If true, Report Title will be displayed in report page footer
        Public Property TitleVisible() As Boolean
            Get
                Return _TitleVisible
            End Get
            Set(ByVal Value As Boolean)
                _TitleVisible = Value
            End Set
        End Property

        'Author
        Public Property Author() As String
            Get
                Return _Author
            End Get
            Set(ByVal Value As String)
                _Author = Value
            End Set
        End Property

        'If true, Author will be displayed in report page footer
        Public Property AuthorVisible() As Boolean
            Get
                Return _AuthorVisible
            End Get
            Set(ByVal Value As Boolean)
                _AuthorVisible = True
            End Set
        End Property

        'Subject
        Public Property Subject() As String
            Get
                Return _Subject
            End Get
            Set(ByVal Value As String)
                _Subject = Value
            End Set
        End Property

        'If true, Subject will be displayed in report page footer
        Public Property SubjectVisible() As Boolean
            Get
                Return _SubjectVisible
            End Get
            Set(ByVal Value As Boolean)
                _SubjectVisible = Value
            End Set
        End Property

        'Gets version of the PDF Generator
        Public ReadOnly Property Version() As String
            Get
                Return "Version 01.01.00"
            End Get
        End Property

        'Version of ITextSharp
        Public ReadOnly Property ITextSharpVersion() As String
            Get
                Dim Result As String = "xx.xx.xx.xx"
                Dim asm As [Assembly] = [Assembly].LoadWithPartialName("iTextSharp")
                If Not asm Is Nothing Then
                    Dim m As Match = Regex.Match(asm.FullName, ".*Version=(\d{1,2}\.\d{1,2}\.\d{1,2}.\d{1,2}).*")
                    If m.Success Then Result = m.Groups(1).Value
                End If
                Return Result
            End Get
        End Property

        'Returns default Creator
        Public ReadOnly Property Creator() As String
            Get
                Return "PDFGenerator " & Version & " (based on iTextSharp ver. " & ITextSharpVersion & " PDF Library)"
            End Get
        End Property

        'DateCreated
        Public Property DateCreated() As DateTime
            Get
                Return _DateCreated
            End Get
            Set(ByVal Value As DateTime)
                _DateCreated = Value
            End Set
        End Property

        'If true, DateCreated will be displayed in report page footer
        Public Property DateCreatedVisible() As Boolean
            Get
                Return _DateCreatedVisible
            End Get
            Set(ByVal Value As Boolean)
                _DateCreatedVisible = Value
            End Set
        End Property

        'Collection of report sections - header, title, label, separater, PDFGrid, etc.
        Public ReadOnly Property Sections() As SectionCollection
            Get
                If _Sections Is Nothing Then
                    _Sections = New SectionCollection
                End If
                Return _Sections
            End Get
        End Property

        'Returns true if PDF content has been rendered
        Public ReadOnly Property IsRendered() As Boolean
            Get
                Return _IsRendered
            End Get
        End Property
#End Region

#Region "Public Methods"

        Public Overridable Sub OnOpenDocument()
            OpenDocument()
        End Sub

        Public Sub OpenDocument()
            PDFWriter.OpenDocument()
        End Sub

        Public Overridable Sub OnDataBind()
            DataBind()
        End Sub

        Public Sub DataBind()
            For Each s As Section In Sections
                s.DataBind()
            Next
        End Sub

        Public Overridable Sub OnRender()
            Render()
        End Sub

        Public Sub Render()
            If Not IsRendered Then
                For Each s As Section In Sections
                    s.Render(PDFWriter)
                Next
                _IsRendered = True
            End If
        End Sub

        Public Sub Flush()
            PDFWriter.PDFWriter.Flush()
            _IsRendered = False
        End Sub

        Public Overridable Sub OnCloseDocument()
            CloseDocument()
        End Sub

        Public Sub CloseDocument()
            PDFWriter.CloseDocument()
        End Sub

        Public Overridable Sub OnOutput()
            Output()
        End Sub

        Public Sub Output()
            Select Case Destination
                Case Destinations.HttpResponse
                    StreamPDF()
                Case Destinations.File
                    SavePDF()
            End Select
        End Sub

        Public Overloads Sub SavePDF(ByVal Path As String)
            OnRender()
            PDFWriter.SavePDF(Path)
        End Sub

        Public Overloads Sub SavePDF()
            SavePDF(OutputFilePath)
        End Sub

        Public Overloads Sub StreamPDF(ByVal Response As System.Web.HttpResponse, ByVal OpenNewWindow As Boolean, ByVal OutputFilePath As String)
            OnRender()
            PDFWriter.StreamPDF(Response, OpenNewWindow, OutputFilePath)
        End Sub

        Public Overloads Sub StreamPDF()
            StreamPDF(Response, OpenNewWindow, OutputFilePath)
        End Sub

        Public Function GetSection(ByVal ID As String) As Section
            For Each s As Section In Sections
                If s.ID = ID Then Return s
            Next
            Return Nothing
        End Function

        Public Function GetSection(ByVal SectionType As SectionTypes) As Section
            For Each s As Section In Sections
                If s.SectionType = SectionType Then Return s
            Next
            Return Nothing
        End Function
#End Region

#Region "Private Methods"

        Private Sub LoadReportDefinitions()
            Dim XMLFileName As String
            Dim XMLFilePath As String = Path.GetDirectoryName(ReportPath)
            Dim XMLDoc As New XmlDocument
            If Not System.Web.HttpContext.Current Is Nothing AndAlso XMLFilePath.Length = 0 Then
                XMLFileName = System.Web.HttpContext.Current.Server.MapPath("./") & ReportPath
            Else
                XMLFileName = ReportPath
            End If
            Try
                XMLDoc.Load(XMLFileName)
            Catch ex As Exception
                Throw New XmlException("Unable to read the report definition file (" & XMLFileName & ") -" & ex.Message)
            End Try
            Dim Nodes As XmlNodeList
            Dim Node As XmlNode
            Dim NodeAttributes As XmlAttributeCollection
            Dim Attribute As XmlAttribute
            Dim RootNode As XmlNode = XMLDoc.LastChild
            If RootNode.Name <> "Document" Then
                Throw New XmlException("Missing XML root node - must be 'Document' not '" & RootNode.Name & "'")
            End If
            _PDFWriter = New PDFWriter
            Try
                Processing = CType([Enum].Parse(GetType(ProcessingMethods), CType(Common.GetNamedAttribute(RootNode, "Processing", "Automatic"), String)), ProcessingMethods)
            Catch ex As Exception
                Throw New XmlException("Processing attribute must be 'Automatic', 'ManualDataBind', 'ManualOutput', or 'Manual'")
            End Try
            Try
                Destination = CType([Enum].Parse(GetType(Destinations), CType(Common.GetNamedAttribute(RootNode, "Destination", "HttpResponse"), String)), Destinations)
            Catch ex As Exception
                Throw New XmlException("Output attribute must be 'Response' or 'File'")
            End Try
            Try
                OpenNewWindow = CType(Common.GetNamedAttribute(RootNode, "OpenNewWindow", True), Boolean)
            Catch ex As Exception
                Throw New XmlException("OpenNewWindow attribute must be 'True' or 'False'")
            End Try
            OutputFilePath = CType(Common.GetNamedAttribute(RootNode, "OutputFilePath", String.Empty), String)

            Dim DocumentHeaderNode As XmlNode = RootNode.Item("DocumentHeader")
            If Not DocumentHeaderNode Is Nothing Then
                Dim AuthorNode As XmlNode = DocumentHeaderNode.Item("Author")
                If Not AuthorNode Is Nothing Then
                    Author = AuthorNode.InnerText
                    Try
                        AuthorVisible = CType(Common.GetNamedAttribute(AuthorNode, "Visible", True), Boolean)
                    Catch ex As Exception
                        Throw New XmlException("Visible attribute of Author node must be 'True' or 'False'")
                    End Try
                    If Author <> String.Empty Then PDFWriter.Author = Author
                End If

                PDFWriter.Creator = Creator

                Dim DateCreatedNode As XmlNode = DocumentHeaderNode.Item("DateCreated")
                If Not DateCreatedNode Is Nothing Then
                    Try
                        DateCreated = DateTime.Parse(DateCreatedNode.InnerText)
                    Catch ex As Exception
                        Throw New XmlException("DateCreated node value is invalid")
                    End Try
                    Try
                        DateCreatedVisible = CType(Common.GetNamedAttribute(DateCreatedNode, "Visible", True), Boolean)
                    Catch ex As Exception
                        Throw New XmlException("Visible attribute of DateCreated node must be 'True' or 'False'")
                    End Try
                    If Not DateCreated.Equals(DateTime.MinValue) Then PDFWriter.DateCreated = DateCreated
                End If

                Dim SubjectNode As XmlNode = DocumentHeaderNode.Item("Subject")
                If Not SubjectNode Is Nothing Then
                    Subject = SubjectNode.InnerText
                    Try
                        SubjectVisible = CType(Common.GetNamedAttribute(SubjectNode, "Visible", True), Boolean)
                    Catch ex As Exception
                        Throw New XmlException("Visible attribute of Subject node must be 'True' or 'False'")
                    End Try
                    If Subject <> String.Empty Then PDFWriter.Subject = Subject
                End If

                Dim TitleNode As XmlNode = DocumentHeaderNode.Item("Title")
                If Not TitleNode Is Nothing Then
                    Title = TitleNode.InnerText
                    Try
                        TitleVisible = CType(Common.GetNamedAttribute(TitleNode, "Visible", True), Boolean)
                    Catch ex As Exception
                        Throw New XmlException("Visible attribute of Title node must be 'True' or 'False'")
                    End Try
                    If Title <> String.Empty Then PDFWriter.Title = Title
                End If
            End If

            Dim ReportNodes As XmlNode = RootNode.Item("Reports")
            If ReportNodes Is Nothing Then
                Throw New XmlException("Document must contain at least one Report node")
            End If

            For Each ReportNode As XmlNode In ReportNodes.ChildNodes
                Dim PageSizeStr As String = CType(Common.GetNamedAttribute(ReportNode, "PageSize", "Letter"), String)
                Select Case PageSizeStr
                    Case "A4"
                        PDFWriter.PageSize = iTextSharp.text.PageSize.A4
                    Case "Letter"
                        PDFWriter.PageSize = iTextSharp.text.PageSize.LETTER
                    Case "Legal"
                        PDFWriter.PageSize = iTextSharp.text.PageSize.LEGAL
                    Case Else
                        Throw New XmlException("PageSize attribute must be 'A4', 'Letter' or 'Legal' - not " & PageSizeStr)
                End Select

                Dim OrientationStr As String = CType(Common.GetNamedAttribute(ReportNode, "Orientation", "Portrait"), String)
                Try
                    PDFWriter.Orientation = CType([Enum].Parse(GetType(Orientation), OrientationStr), Orientation)
                Catch ex As Exception
                    Throw New XmlException("Orientation attribute must be 'Portrait' or 'Landscape' - not " & OrientationStr)
                End Try

                Dim PageColorStr As String = CType(Common.GetNamedAttribute(ReportNode, "PageColor", String.Empty), String)
                If PageColorStr <> String.Empty Then
                    Try
                        Dim wcc As System.Web.UI.WebControls.WebColorConverter = New System.Web.UI.WebControls.WebColorConverter
                        PDFWriter.PageColor = CType(wcc.ConvertFromString(PageColorStr), System.Drawing.Color)
                    Catch ex As Exception
                        Throw New XmlException("Invalid PageColor - " & PageColorStr)
                    End Try
                End If
                Dim Margins As String = CType(Common.GetNamedAttribute(ReportNode, "Margins", String.Empty), String)
                If Margins <> String.Empty Then
                    Try
                        PDFWriter.Margins = QuadUnit.Parse(Margins)
                    Catch ex As Exception
                        Throw New XmlException("Invalid Margins attribute value - " & ex.Message)
                    End Try
                End If
                Nodes = ReportNode.ChildNodes
                For Each Node In Nodes
                    Dim s As Section = Nothing
                    Select Case Node.Name
                        Case "Title"
                            s = New Label(Node)
                            Title = CType(s, Label).Text
                        Case "Header"
                            PDFWriter.SetHeader(New Header(Node))
                        Case "Label"
                            s = New Label(Node)
                        Case "Separator"
                            s = New Separator(Node)
                        Case "NewPage"
                            s = New NewPage
                        Case "PDFGrid"
                            s = New PDFGrid(Node)
                        Case "Footer"
                            PDFWriter.SetFooter(New Footer(Node))
                        Case Else
                            Throw New XmlException("Unrecognized Report Section Type - " & Node.Name)
                    End Select
                    If Not s Is Nothing Then Sections.Add(s)
                Next
            Next
        End Sub

#End Region

#Region "Constructors"
        Public Sub New()
            _PDFWriter = New PDFWriter
        End Sub

        'Creates a new PDF Generator object and loads the report definition from the specified filepath
        Public Sub New(ByVal Filename As String)
            ReportPath = Filename
            LoadReportDefinitions()
            Select Case Processing
                Case ProcessingMethods.Automatic
                    OnOpenDocument()
                    OnDataBind()
                    OnOutput()

                Case ProcessingMethods.ManualDataBind
                    OpenDocument()

                Case ProcessingMethods.ManualOutput
                    OpenDocument()
                    DataBind()

                Case ProcessingMethods.Manual
                    'do nothing as all processing handled in calling code
            End Select
        End Sub

        'Creates a new PDF Generator object from an existing iTextSharp document and uses the provided I/O Stream
        Public Sub New(ByVal PDFDoc As iTextSharp.text.Document, ByVal PDFStream As System.IO.Stream)
            _PDFWriter = New PDFWriter(PDFDoc, PDFStream)
        End Sub

        'Creates an new PDF Generator object with a given PageSize
        Public Sub New(ByVal PageSize As iTextSharp.text.Rectangle)
            _PDFWriter = New PDFWriter(PageSize)
        End Sub

        'Creates a new PDF Generator object with a given PageSize, Orientation, and PageColor (background)
        Public Sub New(ByVal PageSize As iTextSharp.text.Rectangle, ByVal Orientation As PDF.Orientation, ByVal PageColor As System.Drawing.Color)
            _PDFWriter = New PDFWriter(PageSize, Orientation, PageColor)
        End Sub

        'Creates a new PDF Generator object with a given PageSize, Orientation, PageColor, and top/side/bottom margins
        Public Sub New(ByVal PageSize As iTextSharp.text.Rectangle, ByVal Orientation As PDF.Orientation, _
                       ByVal PageColor As System.Drawing.Color, ByVal Margins As QuadUnit)
            _PDFWriter = New PDFWriter(PageSize, Orientation, PageColor, Margins)
        End Sub

#End Region

    End Class
End Namespace
