Imports System.ComponentModel

Namespace WESNet.PDF

    'Class defining the properties of one column of a PDFGrid
    Public Class PDFGridColumn
        Inherits Object

#Region "Private Members"
        Private _DataField As String
        Private _DataFormatString As String
        Private _FooterText As String
        Private _FooterStyle As WESNet.PDF.CellStyle
        Private _HeaderImageURL As String
        Private _HeaderText As String
        Private _HeaderStyle As WESNet.PDF.CellStyle
        Private _ItemStyle As WESNet.PDF.CellStyle
        Private _AlternatingItemStyle As WESNet.PDF.CellStyle
        Private _Owner As WESNet.PDF.PDFGrid
        Private _ShowFooter As Boolean
        Private _ShowHeader As Boolean
        Private _Visible As Boolean
        Private _FieldDescriptor As PropertyDescriptor
        Private _Padding As QuadUnit
#End Region

#Region "Public Properties"

        'String providing the name of the field of the datasource which will be used to
        'populate rows of this column
        Public Property DataField() As String
            Get
                Return _DataField
            End Get
            Set(ByVal Value As String)
                _DataField = Value
            End Set
        End Property

        'String providing format information for the data in this column
        Public Property DataFormatString() As String
            Get
                Return _DataFormatString
            End Get
            Set(ByVal Value As String)
                _DataFormatString = Value
            End Set
        End Property

        'CellStyle defining the style of this column's footer row
        Public ReadOnly Property FooterStyle() As WESNet.PDF.CellStyle
            Get
                If _FooterStyle Is Nothing Then
                    _FooterStyle = New PDF.CellStyle
                End If
                Return _FooterStyle
            End Get
        End Property

        'String providing the text for this column's footer row
        Public Property FooterText() As String
            Get
                Return _FooterText
            End Get
            Set(ByVal Value As String)
                _FooterText = Value
            End Set
        End Property

        'URL of an image to be displayed in this column's header - rendering not yet implemented!
        Public Property HeaderImageURL() As String
            Get
                Return _HeaderImageURL
            End Get
            Set(ByVal Value As String)
                _HeaderImageURL = Value
            End Set
        End Property

        'CellStyle defining the style of the header row of this column
        Public ReadOnly Property HeaderStyle() As WESNet.PDF.CellStyle
            Get
                If _HeaderStyle Is Nothing Then
                    _HeaderStyle = New PDF.CellStyle
                End If
                Return _HeaderStyle
            End Get
        End Property

        'String providing the text of the header row of this column
        Public Property HeaderText() As String
            Get
                Return _HeaderText
            End Get
            Set(ByVal Value As String)
                _HeaderText = Value
            End Set
        End Property

        'CellStyle defining the style of each row of this column's data
        Public ReadOnly Property ItemStyle() As WESNet.PDF.CellStyle
            Get
                If _ItemStyle Is Nothing Then
                    _ItemStyle = New PDF.CellStyle
                End If
                Return _ItemStyle
            End Get
        End Property

        'CellStyle defining the style of each (even numbered) row of this column's data
        Public ReadOnly Property AlternatingItemStyle() As WESNet.PDF.CellStyle
            Get
                If _AlternatingItemStyle Is Nothing Then
                    _AlternatingItemStyle = New PDF.CellStyle
                End If
                Return _AlternatingItemStyle
            End Get
        End Property

        'Owner of the column - a PDFGrid object
        Protected ReadOnly Property Owner() As WESNet.PDF.PDFGrid
            Get
                Return _Owner
            End Get
        End Property

        'True if this column's footer is to be shown
        Public Property ShowFooter() As Boolean
            Get
                Return _ShowFooter
            End Get
            Set(ByVal Value As Boolean)
                _ShowFooter = Value
            End Set
        End Property

        'True if this column's header is to be shown
        Public Property ShowHeader() As Boolean
            Get
                Return _ShowHeader
            End Get
            Set(ByVal Value As Boolean)
                _ShowHeader = Value
            End Set
        End Property

        'True if the entire column is to be shown
        Public Property Visible() As Boolean
            Get
                Return _Visible
            End Get
            Set(ByVal Value As Boolean)
                _Visible = Value
            End Set
        End Property

        'Padding to be added to the top, right, bottom, and left of each cell in this column.
        'Due to limitations of iTextSharp, the use of top and bottom padding is unreliable
        Public Property Padding() As QuadUnit
            Get
                Return _Padding
            End Get
            Set(ByVal Value As QuadUnit)
                _Padding = Value
            End Set
        End Property
#End Region

#Region "Private/Protected/Friend Properties"

        Friend Property FieldDescriptor() As PropertyDescriptor
            Get
                Return _FieldDescriptor
            End Get
            Set(ByVal Value As PropertyDescriptor)
                _FieldDescriptor = Value
            End Set
        End Property

#End Region

#Region "Private/Protected/Friend Methods"

        Friend Sub SetOwner(ByVal owner As WESNet.PDF.PDFGrid)
            _Owner = owner
        End Sub

#End Region

#Region "Public Methods"
        Public Overridable Function FormatDataValue(ByVal DataValue As Object) As String
            Dim FormattedValue As String = String.Empty
            If Not DataValue Is Nothing AndAlso Not DataValue Is DBNull.Value Then
                If DataFormatString Is Nothing OrElse DataFormatString.Length = 0 Then
                    FormattedValue = Convert.ToString(DataValue)
                Else
                    FormattedValue = String.Format(DataFormatString, DataValue)
                End If
            End If
            Return FormattedValue
        End Function

        Public Overridable Function GetFormattedValue(ByVal DataRowObj As Object) As String
            If FieldDescriptor Is Nothing Then
                Return String.Empty
            Else
                Return FormatDataValue(FieldDescriptor.GetValue(DataRowObj))
            End If
        End Function
#End Region

#Region "Constructors"

        Public Sub New()

        End Sub

        Public Sub New(ByVal DataField As String)
            Me.New(DataField, DataField)
        End Sub

        Public Sub New(ByVal DataField As String, ByVal HeaderText As String)
            _DataField = DataField
            _HeaderText = HeaderText
        End Sub

        Public Sub New(ByVal DataField As String, ByVal HeaderText As String, ByVal FooterText As String)
            _DataField = DataField
            _HeaderText = HeaderText
            _FooterText = FooterText
        End Sub
#End Region

    End Class
End Namespace
