Imports System.ComponentModel

Namespace WESNet.PDF

    'Defines a custom collection of PDFGridColumns

    <DefaultProperty("Item")> _
    Public Class PDFGridColumnCollection
        Implements ICollection, IEnumerable

        Private _Owner As PDFGrid
        Private _Columns As ArrayList

        Public ReadOnly Property Item(ByVal index As Integer) As PDFGridColumn
            Get
                Return CType(_Columns(index), PDFGridColumn)
            End Get
        End Property

        Public Sub Add(ByVal column As PDFGridColumn)
            AddAt(-1, column)
        End Sub

        Public Sub AddAt(ByVal index As Integer, ByVal column As PDFGridColumn)
            If index = -1 Then
                _Columns.Add(column)
            Else
                _Columns.Insert(index, column)
            End If
            column.SetOwner(_Owner)
        End Sub

        Public Function Add(ByVal DataField As String) As PDFGridColumn
            Return Add(DataField, DataField)
        End Function

        Public Function Add(ByVal DataField As String, ByVal HeaderText As String) As PDFGridColumn
            Dim column As New PDFGridColumn(DataField, HeaderText)
            AddAt(-1, column)
            Return column
        End Function

        Public Sub Clear()
            _Columns.Clear()
        End Sub

        Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements ICollection.CopyTo
            Dim enumerator1 As IEnumerator = Me.GetEnumerator
            Do While enumerator1.MoveNext
                array.SetValue(enumerator1.Current, index)
                index += 1
            Loop
        End Sub

        Public ReadOnly Property Count() As Integer Implements ICollection.Count
            Get
                Return _Columns.Count
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return _Columns.GetEnumerator
        End Function

        Public Function IndexOf(ByVal column As PDFGridColumn) As Integer
            If Not column Is Nothing Then
                Return _Columns.IndexOf(column)
            Else
                Return -1
            End If
        End Function

        Public Sub Remove(ByVal column As WESNet.PDF.PDFGridColumn)
            Dim i As Integer = IndexOf(column)
            If i >= 0 Then RemoveAt(i)
        End Sub

        Public Sub RemoveAt(ByVal index As Integer)
            If index < 0 Or index >= Count() Then
                Throw New ArgumentOutOfRangeException("Index of column to be removed is out of range")
            Else
                _Columns.RemoveAt(index)
            End If
        End Sub

        Public Sub New(ByVal Owner As PDFGrid)
            _Owner = Owner
            _Columns = New ArrayList
        End Sub

        Public Sub New(ByVal Owner As PDFGrid, ByVal Columns As ArrayList)
            _Owner = Owner
            _Columns = Columns
        End Sub

        Public ReadOnly Property IsReadOnly() As Boolean
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
            Get
                Return Me
            End Get
        End Property

    End Class
End Namespace
