Imports System.ComponentModel

Namespace WESNet.PDF

    'Defines a custom collection of PDFItems

    <DefaultProperty("Item")> _
    Public Class PDFItemCollection
        Implements ICollection, IEnumerable

        Private _PDFItems As ArrayList
        Private _Owner As Section

        Public ReadOnly Property Item(ByVal index As Integer) As PDFItem
            Get
                Return CType(_PDFItems(index), PDFItem)
            End Get
        End Property

        Public Sub Add(ByVal PDFItem As PDFItem)
            AddAt(-1, PDFItem)
        End Sub

        Public Sub AddAt(ByVal index As Integer, ByVal PDFItem As PDFItem)
            If index = -1 Then
                _PDFItems.Add(PDFItem)
            Else
                _PDFItems.Insert(index, PDFItem)
            End If
        End Sub

        Public Sub Clear()
            _PDFItems.Clear()
        End Sub

        Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements ICollection.CopyTo
            Dim enumerator1 As IEnumerator = Me.GetEnumerator
            Do While enumerator1.MoveNext
                array.SetValue(enumerator1.Current, index)
                index += 1
            Loop
        End Sub

        Public ReadOnly Property Count() As Integer Implements ICollection.Count
            Get
                Return _PDFItems.Count
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return _PDFItems.GetEnumerator
        End Function

        Public Function IndexOf(ByVal PDFItem As PDFItem) As Integer
            If Not PDFItem Is Nothing Then
                Return _PDFItems.IndexOf(PDFItem)
            Else
                Return -1
            End If
        End Function

        Public Sub Remove(ByVal PDFItem As PDFItem)
            Dim i As Integer = IndexOf(PDFItem)
            If i >= 0 Then RemoveAt(i)
        End Sub

        Public Sub RemoveAt(ByVal index As Integer)
            If index < 0 Or index >= Count() Then
                Throw New ArgumentOutOfRangeException("Index of PDFItem to be removed is out of range")
            Else
                _PDFItems.RemoveAt(index)
            End If
        End Sub

        Public ReadOnly Property IsReadOnly() As Boolean
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
            Get
                Return Me
            End Get
        End Property

        Public ReadOnly Property Owner() As Section
            Get
                Return _Owner
            End Get
        End Property

#Region "Constructors"
        Public Sub New(ByVal Owner As Section)
            _Owner = Owner
            _PDFItems = New ArrayList
        End Sub

        Public Sub New(ByVal Owner As Section, ByVal PDFItems As ArrayList)
            _Owner = Owner
            _PDFItems = PDFItems
        End Sub
#End Region
    End Class
End Namespace
