Imports System.ComponentModel
Imports System.Globalization
Imports System.Text
Imports System.Web.UI.WebControls

Namespace WESNet

    'The QuadUnit class is to define border and margin values for cells and the document
    'For example, "0.5in, 1.0in, 0.75in, 1.0in" could be used to define the Top, Right,
    'Bottom, and Left margins of the document.

    <TypeConverter(GetType(QuadUnitConverter))> _
    Public Class QuadUnit

#Region "Private Members"
        Private _Top As Unit
        Private _Right As Unit
        Private _Bottom As Unit
        Private _Left As Unit
        Private _SetBits As Integer
#End Region

#Region "Private Constants"
        Private Const PROP_TOP As Integer = &H80
        Private Const PROP_RIGHT As Integer = &H100
        Private Const PROP_BOTTOM As Integer = &H200
        Private Const PROP_LEFT As Integer = &H400
#End Region

#Region "Public Shared Member"
        Public Shared ReadOnly Empty As QuadUnit = New QuadUnit
        Public Shared ReadOnly Zero As QuadUnit = New QuadUnit(Unit.Pixel(0), Unit.Pixel(0), Unit.Pixel(0), Unit.Pixel(0))
#End Region

#Region "Public Properties"

        Public Property Top() As Unit
            Get
                If IsSet(PROP_TOP) Then
                    Return _Top
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                _Top = Value
                SetBit(PROP_TOP, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Right() As Unit
            Get
                If IsSet(PROP_RIGHT) Then
                    Return _Right
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                _Right = Value
                SetBit(PROP_RIGHT, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Bottom() As Unit
            Get
                If IsSet(PROP_BOTTOM) Then
                    Return _Bottom
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                _Bottom = Value
                SetBit(PROP_BOTTOM, Not Value.IsEmpty)
            End Set
        End Property

        Public Property Left() As Unit
            Get
                If IsSet(PROP_LEFT) Then
                    Return _Left
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                _Left = Value
                SetBit(PROP_LEFT, Not Value.IsEmpty)
            End Set
        End Property

        Public ReadOnly Property IsEmpty() As Boolean
            Get
                Return _SetBits = 0
            End Get
        End Property

#End Region

#Region "Private/Protected Methods"

        Private Overloads Sub SetBit(ByVal bitMask As Integer)
            _SetBits = _SetBits Or bitMask
        End Sub

        Private Sub ClearBit(ByVal bitMask As Integer)
            _SetBits = _SetBits And (Not bitMask)
        End Sub

        Private Overloads Sub SetBit(ByVal bitMask As Integer, ByVal Condition As Boolean)
            If Condition Then
                SetBit(bitMask)
            Else
                ClearBit(bitMask)
            End If
        End Sub

        Private Function IsSet(ByVal bitMask As Integer) As Boolean
            Return (_SetBits And bitMask) <> 0
        End Function
#End Region

#Region "Public Methods"
        Public Shadows Function Equals(ByVal obj As Object) As Boolean
            If obj Is Nothing Then
                Return False
            Else
                Dim QuadUnitObj As QuadUnit = CType(obj, QuadUnit)
                Return _Top.Equals(QuadUnitObj.Top) And _
                       _Right.Equals(QuadUnitObj.Right) And _
                       _Bottom.Equals(QuadUnitObj.Bottom) And _
                       _Left.Equals(QuadUnitObj.Left)
            End If
        End Function

        Public Overloads Function ToString(ByVal culture As CultureInfo) As String
            If IsEmpty Then Return String.Empty
            Dim SB As New StringBuilder(_Top.ToString(culture))
            If _Top.Equals(_Bottom) And _Right.Equals(_Left) And _Top.Equals(_Right) _
               Then Return SB.ToString
            SB.Append(" ")
            SB.Append(_Right.ToString(culture))
            If _Right.Equals(_Left) Then Return SB.ToString()
            SB.Append(" ")
            SB.Append(_Bottom.ToString(culture))
            SB.Append(" ")
            SB.Append(_Left.ToString(culture))
            Return SB.ToString()
        End Function

        Public Overloads Overrides Function ToString() As String
            Return ToString(CultureInfo.CurrentCulture)
        End Function

        Public Function ToArray() As Unit()
            Dim al As New ArrayList
            If IsEmpty Then Return Nothing
            al.Add(_Top)
            If _Top.Equals(Bottom) And _Right.Equals(_Left) And _Top.Equals(_Right) _
                Then Return CType(al.ToArray(GetType(Unit)), Unit())
            al.Add(_Right)
            If _Right.Equals(_Left) Then Return CType(al.ToArray(GetType(Unit)), Unit())
            al.Add(_Bottom)
            al.Add(_Left)
            Return CType(al.ToArray(GetType(Unit)), Unit())
        End Function

        Public Sub CopyFrom(ByVal b As QuadUnit)
            If Not b.IsEmpty Then
                With b
                    If .IsSet(PROP_TOP) And Not .Top.IsEmpty Then
                        Top = .Top
                    End If
                    If .IsSet(PROP_RIGHT) And Not .Right.IsEmpty Then
                        Right = .Right
                    End If
                    If .IsSet(PROP_BOTTOM) And Not .Bottom.IsEmpty Then
                        Bottom = .Bottom
                    End If
                    If .IsSet(PROP_LEFT) And Not .Left.IsEmpty Then
                        Left = .Left
                    End If
                End With
            End If
        End Sub

        Public Sub MergeWith(ByVal b As QuadUnit)
            If Not b.IsEmpty Then
                If IsEmpty Then
                    CopyFrom(b)
                Else
                    With b
                        If .IsSet(PROP_TOP) And (Not IsSet(PROP_TOP) Or Top.IsEmpty) Then
                            Top = .Top
                        End If
                        If .IsSet(PROP_RIGHT) And (Not IsSet(PROP_RIGHT) Or Right.IsEmpty) Then
                            Right = .Right
                        End If
                        If .IsSet(PROP_BOTTOM) And (Not IsSet(PROP_BOTTOM) Or Bottom.IsEmpty) Then
                            Bottom = .Bottom
                        End If
                        If .IsSet(PROP_LEFT) And (Not IsSet(PROP_LEFT) Or Left.IsEmpty) Then
                            Left = .Left
                        End If
                    End With
                End If
            End If
        End Sub
#End Region

#Region "Public Shared Methods"
        Public Overloads Shared Function Parse(ByVal s As String, ByVal culture As CultureInfo) As QuadUnit
            Return New QuadUnit(s, culture, UnitType.Pixel)
        End Function

        Public Overloads Shared Function Parse(ByVal s As String) As QuadUnit
            Return New QuadUnit(s, CultureInfo.CurrentCulture, UnitType.Pixel)
        End Function

        Public Shared Function Point(ByVal value As Single) As QuadUnit
            Return New QuadUnit(Unit.Point(value))
        End Function

        Public Shared Function Pixel(ByVal value As Single) As QuadUnit
            Return New QuadUnit(Unit.Pixel(value))
        End Function
#End Region

#Region "Public Constructors"
        Public Sub New()
            'Top = Unit.Pixel(0.0!)
            'Right = Unit.Pixel(0.0!)
            'Bottom = Unit.Pixel(0.0!)
            'Left = Unit.Pixel(0.0!)
        End Sub

        Public Sub New(ByVal All As Unit)
            Top = All
            Right = All
            Bottom = All
            Left = All
        End Sub

        Public Sub New(ByVal TopBottom As Unit, ByVal RightLeft As Unit)
            Top = TopBottom
            Right = RightLeft
            Bottom = TopBottom
            Left = RightLeft
        End Sub

        Public Sub New(ByVal Top As Unit, ByVal Right As Unit, ByVal Bottom As Unit, ByVal Left As Unit)
            Me.Top = Top
            Me.Right = Right
            Me.Bottom = Bottom
            Me.Left = Left
        End Sub

        Public Sub New(ByVal s As String)
            Me.New(s, CultureInfo.CurrentCulture, UnitType.Pixel)
        End Sub

        Public Sub New(ByVal s As String, ByVal culture As CultureInfo)
            Me.New(s, culture, UnitType.Pixel)
        End Sub

        Public Sub New(ByVal s As String, ByVal culture As CultureInfo, ByVal defaulttype As UnitType)
            If culture Is Nothing Then
                culture = CultureInfo.CurrentCulture
            End If
            If s Is Nothing Then
                'Top = New Unit(0.0!, defaulttype)
                'Right = _Top
                'Bottom = _Top
                'Left = _Top
                Exit Sub
            End If
            s = s.Trim.ToLower(CultureInfo.InvariantCulture)
            If s.Length = 0 Then
                'Top = New Unit(0.0!, defaulttype)
                'Right = _Top
                'Bottom = _Top
                'Left = _Top
                Exit Sub
            End If
            Dim Units As String() = s.Split(" "c)
            Select Case Units.Length
                Case 1
                    Top = Unit.Parse(Units(0), culture)
                    Right = _Top
                    Bottom = _Top
                    Left = _Top
                Case 2
                    Top = Unit.Parse(Units(0), culture)
                    Right = Unit.Parse(Units(1), culture)
                    Bottom = _Top
                    Left = _Right
                Case 3
                    Throw New ArgumentException("Quad unit string must contain 1, 2, or 4 unit values")
                Case 4
                    Top = Unit.Parse(Units(0), culture)
                    Right = Unit.Parse(Units(1), culture)
                    Bottom = Unit.Parse(Units(2), culture)
                    Left = Unit.Parse(Units(3), culture)
            End Select
        End Sub
#End Region

    End Class
End Namespace
