Imports System.ComponentModel
Imports System.Globalization
Imports System.Web.UI.WebControls

Namespace WESNet
    Public Class QuadUnitConverter
        Inherits TypeConverter
        Public Overloads Overrides Function CanConvertFrom(ByVal context As ITypeDescriptorContext, ByVal sourceType As Type) As Boolean
            If (sourceType Is GetType(String)) Or sourceType Is GetType(Unit()) Then
                Return True
            End If
            Return MyBase.CanConvertFrom(context, sourceType)
        End Function

        Public Overloads Overrides Function ConvertFrom(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object) As Object
            If value Is Nothing Then Return Nothing
            If TypeOf value Is String Then
                Dim text1 As String = CType(value, String).Trim()
                If text1.Length = 0 Then Return QuadUnit.Empty
                Return QuadUnit.Parse(text1)
            End If
            If TypeOf value Is Unit() Then
                Dim units As Unit() = CType(value, Unit())
                Select Case units.Length - 1
                    Case 1
                        Return New QuadUnit(units(0))
                    Case 2
                        Return New QuadUnit(units(0), units(1))
                    Case 3
                        Throw New ArgumentException("Quad unit must contain 1, 2, or 4 unit values")
                    Case Else
                        Return New QuadUnit(units(0), units(1), units(2), units(3))
                End Select
            End If
            Return MyBase.ConvertFrom(context, culture, value)
        End Function

        Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
            If value Is Nothing Then Return Nothing
            Dim quadunit1 As QuadUnit = CType(value, QuadUnit)
            If destinationType Is GetType(String) Then
                If Not quadunit1.IsEmpty Then
                    Return quadunit1.ToString(culture)
                Else
                    Return String.Empty
                End If
            End If
            If destinationType Is GetType(Unit()) Then
                If Not quadunit1.IsEmpty Then
                    Return quadunit1.ToArray()
                Else
                    Return Nothing
                End If
            End If
            Return MyBase.ConvertTo(context, culture, value, destinationType)
        End Function
    End Class
End Namespace
