Imports System.ComponentModel

Namespace WESNet.PDF

    'A custom collection of document Sections

    <DefaultProperty("Item")> _
    Public Class SectionCollection
        Implements ICollection, IEnumerable
        Private _Sections As ArrayList

        Public ReadOnly Property Item(ByVal index As Integer) As Section
            Get
                Return CType(_Sections(index), Section)
            End Get
        End Property

        Public Sub Add(ByVal s As Section)
            AddAt(-1, s)
        End Sub

        Public Sub AddAt(ByVal index As Integer, ByVal s As Section)
            If index = -1 Then
                _Sections.Add(s)
            Else
                _Sections.Insert(index, s)
            End If
        End Sub

        Public Sub Clear()
            _Sections.Clear()
        End Sub

        Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements ICollection.CopyTo
            Dim enumerator1 As IEnumerator = Me.GetEnumerator
            Do While enumerator1.MoveNext
                array.SetValue(enumerator1.Current, index)
                index += 1
            Loop
        End Sub

        Public ReadOnly Property Count() As Integer Implements ICollection.Count
            Get
                Return _Sections.Count
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return _Sections.GetEnumerator
        End Function

        Public Function IndexOf(ByVal s As Section) As Integer
            If Not s Is Nothing Then
                Return _Sections.IndexOf(s)
            Else
                Return -1
            End If
        End Function

        Public Sub Remove(ByVal s As Section)
            Dim i As Integer = IndexOf(s)
            If i >= 0 Then RemoveAt(i)
        End Sub

        Public Sub RemoveAt(ByVal index As Integer)
            If index < 0 Or index >= Count() Then
                Throw New ArgumentOutOfRangeException("Index of section to be removed is out of range")
            Else
                _Sections.RemoveAt(index)
            End If
        End Sub

        Public ReadOnly Property IsReadOnly() As Boolean
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
            Get
                Return Me
            End Get
        End Property

#Region "Constructors"
        Public Sub New()
            _Sections = New ArrayList
        End Sub

        Public Sub New(ByVal Sections As ArrayList)
            _Sections = Sections
        End Sub
#End Region

    End Class
End Namespace
