Imports System.ComponentModel
Imports WESNet

Namespace WESNet.PDF

    'Adds the Padding, HorizontalAlign, Leading, SpacingAfter and SpacingBefore
    'properties to the base Style class

    <TypeConverter(GetType(ExpandableObjectConverter))> _
   Public Class SectionStyle
        Inherits Style

#Region "Private Members"
        Private _Padding As QuadUnit
        Private _Leading As Unit
        Private _SpacingAfter As Unit
        Private _SpacingBefore As Unit
        Private _HorizontalAlign As System.Web.UI.WebControls.HorizontalAlign
#End Region

#Region "Public Properties"

        Public ReadOnly Property Padding() As QuadUnit
            Get
                If _Padding Is Nothing OrElse _Padding.IsEmpty Then
                    _Padding = New QuadUnit
                End If
                Return _Padding
            End Get
        End Property

        Public Property HorizontalAlign() As System.Web.UI.WebControls.HorizontalAlign
            Get
                If IsSet(PROP_HORIZALIGN) Then
                    Return _HorizontalAlign
                Else
                    Return Web.UI.WebControls.HorizontalAlign.NotSet
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.HorizontalAlign)
                If Value < Web.UI.WebControls.HorizontalAlign.NotSet Or Value > Web.UI.WebControls.HorizontalAlign.Justify Then
                    Throw New ArgumentOutOfRangeException("Invalid value specified for HorizontalAlign")
                End If
                _HorizontalAlign = Value
                SetBit(PROP_HORIZALIGN)
            End Set
        End Property

        Public Property Leading() As Unit
            Get
                If IsSet(PROP_LEADING) Then
                    Return _Leading
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                _Leading = Value
                SetBit(PROP_LEADING)
            End Set
        End Property

        Public Property SpacingAfter() As Unit
            Get
                If IsSet(PROP_SPACINGAFTER) Then
                    Return _SpacingAfter
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                If Value.Value < 0 Then
                    Throw New ArgumentOutOfRangeException("Spacing after value cannot be negative")
                Else
                    _SpacingAfter = Value
                    SetBit(PROP_SPACINGAFTER)
                End If
            End Set
        End Property

        Public Property SpacingBefore() As Unit
            Get
                If IsSet(PROP_SPACINGBEFORE) Then
                    Return _SpacingBefore
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                If Value.Value < 0 Then
                    Throw New ArgumentOutOfRangeException("Spacing Before value cannot be negative")
                Else
                    _SpacingBefore = Value
                    SetBit(PROP_SPACINGBEFORE)
                End If
            End Set
        End Property
#End Region

#Region "Public Methods"

        Public Overrides Sub CopyFrom(ByVal s As PDF.Style)
            If Not s Is Nothing AndAlso Not s.IsEmpty Then
                MyBase.CopyFrom(s)
                If TypeOf (s) Is SectionStyle Then
                    Dim s1 As SectionStyle = CType(s, SectionStyle)
                    With s1
                        Padding.CopyFrom(.Padding)
                        If .IsSet(PROP_LEADING) And Not .Leading.IsEmpty Then
                            Leading = .Leading
                        End If
                        If .IsSet(PROP_SPACINGAFTER) And Not .SpacingAfter.IsEmpty Then
                            SpacingAfter = .SpacingAfter
                        End If
                        If .IsSet(PROP_SPACINGBEFORE) And Not .SpacingBefore.IsEmpty Then
                            SpacingBefore = .SpacingBefore
                        End If
                        If .IsSet(PROP_HORIZALIGN) And .HorizontalAlign <> Web.UI.WebControls.HorizontalAlign.NotSet Then
                            HorizontalAlign = .HorizontalAlign
                        End If
                    End With
                End If
            End If
        End Sub

        Public Overrides Sub MergeWith(ByVal s As PDF.Style)
            If Not s Is Nothing AndAlso Not s.IsEmpty Then
                If IsEmpty Then
                    CopyFrom(s)
                Else
                    MyBase.MergeWith(s)
                    If TypeOf (s) Is SectionStyle Then
                        Dim s1 As PDF.SectionStyle = CType(s, PDF.SectionStyle)
                        With s1
                            Padding.MergeWith(.Padding)
                            If .IsSet(PROP_HORIZALIGN) And ((Not IsSet(PROP_HORIZALIGN)) Or HorizontalAlign = Web.UI.WebControls.HorizontalAlign.NotSet) Then
                                HorizontalAlign = .HorizontalAlign
                            End If
                        End With
                    End If
                End If
            End If
        End Sub
#End Region

    End Class
End Namespace
