Imports System.ComponentModel

Namespace WESNet.PDF

    'Encapsulates the properties of BackColor, Border, Font, ForeColor, Height and Width
    'into a Style object

    <TypeConverter(GetType(ExpandableObjectConverter))> _
   Public Class Style
        Inherits Object

#Region "Private Members"
        Private _BackColor As System.Drawing.Color
        Private _Border As Border
        Private _Font As PDF.FontInfo
        Friend _FontBold As Boolean
        Friend _FontItalic As Boolean
        Friend _FontNames As String()
        Friend _FontOverline As Boolean
        Friend _FontSize As System.Web.UI.WebControls.FontUnit
        Friend _FontStrikeout As Boolean
        Friend _FontUnderline As Boolean
        Private _ForeColor As System.Drawing.Color
        Private _Height As Unit
        Private _Width As Unit
        Private _SetBits As Integer
#End Region

#Region "Private Constants"

        'Note - These bitmap constants must be unique and be powers of 2
        Protected Const PROP_BACKCOLOR As Integer = &H1
        Protected Const PROP_BORDER As Integer = &H2
        Protected Const PROP_CELLSPACING As Integer = &H4
        Protected Const PROP_CELLPADDING As Integer = &H8

        'Reserved for font property constants &H8 through &H200

        Protected Const PROP_FORECOLOR As Integer = &H400
        Protected Const PROP_GRIDLINES As Integer = &H800
        Protected Const PROP_HEIGHT As Integer = &H1000
        Protected Const PROP_HORIZALIGN As Integer = &H2000
        Protected Const PROP_LEADING As Integer = &H4000
        Protected Const PROP_PADDING As Integer = &H8000
        Protected Const PROP_SPACINGAFTER As Integer = &H10000
        Protected Const PROP_SPACINGBEFORE As Integer = &H20000
        Protected Const PROP_VERTALIGN As Integer = &H40000
        Protected Const PROP_WIDTH As Integer = &H80000
        Protected Const PROP_WRAP As Integer = &H100000

#End Region

#Region "Public Properties"
        '<TypeConverter(GetType(System.Drawing.ColorConverter))> _
        Public Property BackColor() As System.Drawing.Color
            Get
                If IsSet(PROP_BACKCOLOR) Then
                    Return _BackColor
                Else
                    Return System.Drawing.Color.Empty
                End If
            End Get
            Set(ByVal Value As System.Drawing.Color)
                _BackColor = Value
                SetBit(PROP_BACKCOLOR)
            End Set
        End Property

        Public ReadOnly Property Border() As Border
            Get
                If _Border Is Nothing OrElse _Border.IsEmpty Then
                    _Border = New Border
                End If
                Return _Border
            End Get
        End Property

        Public ReadOnly Property Font() As PDF.FontInfo
            Get
                If _Font Is Nothing Then
                    _Font = New PDF.FontInfo(Me)
                End If
                Return _Font
            End Get
        End Property

        Public Property ForeColor() As System.Drawing.Color
            Get
                If IsSet(PROP_FORECOLOR) Then
                    Return _ForeColor
                Else
                    Return System.Drawing.Color.Empty
                End If
            End Get
            Set(ByVal Value As System.Drawing.Color)
                _ForeColor = Value
                SetBit(PROP_FORECOLOR)
            End Set
        End Property

        Public Property Height() As Unit
            Get
                If IsSet(PROP_HEIGHT) Then
                    Return _Height
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                If Value.Value < 0 Then
                    Throw New ArgumentOutOfRangeException("Height value must be >=0")
                Else
                    _Height = Value
                    SetBit(PROP_HEIGHT)
                End If
            End Set
        End Property

        Public Property Width() As Unit
            Get
                If IsSet(PROP_WIDTH) Then
                    Return _Width
                Else
                    Return Unit.Empty
                End If
            End Get
            Set(ByVal Value As Unit)
                If Value.Value < 0 Then
                    Throw New ArgumentOutOfRangeException("Width value msut be >=0")
                Else
                    _Width = Value
                    SetBit(PROP_WIDTH)
                End If
            End Set
        End Property
#End Region

#Region "Private/Protected Properties"

        Protected Friend Overridable ReadOnly Property IsEmpty() As Boolean
            Get
                Return _SetBits = 0
            End Get
        End Property

#End Region

#Region "Private/Protected Methods"

        Protected Friend Overridable Sub SetBit(ByVal bitMask As Integer)
            _SetBits = _SetBits Or bitMask
        End Sub

        Friend Function IsSet(ByVal bitMask As Integer) As Boolean
            Return (_SetBits And bitMask) <> 0
        End Function
#End Region

#Region "Public Methods"

        Public Overridable Sub CopyFrom(ByVal s As PDF.Style)
            If Not s Is Nothing AndAlso Not s.IsEmpty Then
                With s
                    If .IsSet(PROP_HEIGHT) And Not .Height.IsEmpty Then
                        Height = .Height
                    End If
                    Border.CopyFrom(.Border)
                    Font.CopyFrom(.Font)
                    If .IsSet(PROP_FORECOLOR) And Not .ForeColor.IsEmpty Then
                        ForeColor = .ForeColor
                    End If
                    If .IsSet(PROP_BACKCOLOR) And Not .BackColor.IsEmpty Then
                        BackColor = .BackColor
                    End If
                    If .IsSet(PROP_WIDTH) And Not .Width.IsEmpty Then
                        Width = .Width
                    End If
                End With
            End If
        End Sub

        Public Overridable Sub MergeWith(ByVal s As PDF.Style)
            If Not s Is Nothing AndAlso Not s.IsEmpty Then
                If IsEmpty Then
                    CopyFrom(s)
                Else
                    With s
                        If .IsSet(PROP_HEIGHT) And (Not IsSet(PROP_HEIGHT) Or Height.IsEmpty) Then
                            Height = .Height
                        End If
                        Border.MergeWith(.Border)
                        Font.MergeWith(.Font)
                        If .IsSet(PROP_FORECOLOR) And (Not IsSet(PROP_FORECOLOR) Or ForeColor.IsEmpty) Then
                            ForeColor = .ForeColor
                        End If
                        If .IsSet(PROP_BACKCOLOR) And (Not IsSet(PROP_BACKCOLOR) Or BackColor.IsEmpty) Then
                            BackColor = .BackColor
                        End If
                        If .IsSet(PROP_WIDTH) And (Not IsSet(PROP_WIDTH) Or Width.IsEmpty) Then
                            Width = .Width
                        End If
                    End With
                End If
            End If
        End Sub
#End Region


    End Class
End Namespace
