Namespace WESNet.PDF

    'Represents on cell in a table row

    Public Class TableCell

#Region "Private Members"
        Private _ColumnSpan As Integer
        Private _RowSpan As Integer
        Private _Style As WESNet.PDF.CellStyle
        Private _Cell As iTextSharp.text.Cell
        Private _Text As String
        Private _Visible As Boolean = True
        Private _Owner As WESNet.PDF.TableRow
#End Region

#Region "Public Properties"

        'Number of table columns spanned by this cell
        Public Property ColumnSpan() As Integer
            Get
                Return _ColumnSpan
            End Get
            Set(ByVal Value As Integer)
                _ColumnSpan = Value
            End Set
        End Property

        'Horizontal alignment of the contents of this table cell
        Public Property HorizontalAlign() As System.Web.UI.WebControls.HorizontalAlign
            Get
                If StyleCreated Then
                    Return Style.HorizontalAlign
                Else
                    Return Web.UI.WebControls.HorizontalAlign.NotSet
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.HorizontalAlign)
                Style.HorizontalAlign = Value
            End Set
        End Property

        'Access the owner of this table cell - a TableRow object
        Public ReadOnly Property Owner() As WESNet.PDF.TableRow
            Get
                Return _Owner
            End Get
        End Property

        'Accesses the underlying iTextSharp.text.Cell represented by this table cell
        Public Property Cell() As iTextSharp.text.Cell
            Get
                Return _Cell
            End Get
            Set(ByVal Value As iTextSharp.text.Cell)
                _Cell = Value
            End Set
        End Property

        'The numbe of table rows spanned by this table cell
        Public Property RowSpan() As Integer
            Get
                Return _RowSpan
            End Get
            Set(ByVal Value As Integer)
                _RowSpan = Value
            End Set
        End Property

        'The CellStyle of this table cell - usually inherited from the cell's column ItemStyle
        Public ReadOnly Property Style() As WESNet.PDF.CellStyle
            Get
                If _Style Is Nothing Then
                    _Style = New PDF.CellStyle
                End If
                Return _Style
            End Get
        End Property

        'Returns true if the table cell's Style has been instantiated
        Public ReadOnly Property StyleCreated() As Boolean
            Get
                Return Not _Style Is Nothing
            End Get
        End Property

        'The text of the table cell
        Public Property Text() As String
            Get
                Return _Text
            End Get
            Set(ByVal Value As String)
                _Text = Value
            End Set
        End Property

        'True if the table cell contents are visible
        Public Property Visible() As Boolean
            Get
                Return _Visible
            End Get
            Set(ByVal Value As Boolean)
                _Visible = True
            End Set
        End Property

        'The vertical alignment of the contents of the table cell
        'Due to limitations of iTextSharp, this property does not reliably set the vertical alignment
        'of the table cell
        Public Property VerticalAlign() As System.Web.UI.WebControls.VerticalAlign
            Get
                If StyleCreated Then
                    Return Style.VerticalAlign
                Else
                    Return Web.UI.WebControls.VerticalAlign.NotSet
                End If
            End Get
            Set(ByVal Value As System.Web.UI.WebControls.VerticalAlign)
                Style.VerticalAlign = Value
            End Set
        End Property

        'If true (default), the contents of the tablecell will wrap lines its contents that
        'exceed the cell's width
        Public Property Wrap() As Boolean
            Get
                If StyleCreated Then
                    Return Style.Wrap
                Else
                    Return True
                End If
            End Get
            Set(ByVal Value As Boolean)
                Style.Wrap = Value
            End Set
        End Property
#End Region

#Region "Constructors"
        Public Sub New(ByVal Owner As WESNet.PDF.TableRow)
            _Owner = Owner
        End Sub
#End Region

#Region "Private/Protected/Friend Methods"
        Friend Sub SetOwner(ByVal Owner As WESNet.PDF.TableRow)
            _Owner = Owner
        End Sub
#End Region

#Region "Public Methods"

        Public Sub AddToTable(ByVal Table As iTextSharp.text.Table)
            If Style.IsEmpty And ColumnSpan <= 0 And RowSpan <= 0 Then
                Table.AddCell(Text)
            Else
                'Dim Chunk As iTextSharp.text.Chunk
                Dim Paragraph As iTextSharp.text.Paragraph
                Cell = New iTextSharp.text.Cell
                With Cell
                    'Note: Fixed cell width/height not supported in this
                    'version of iTextSharp and are ignored for now

                    If Style.Border.IsEmpty Then
                        .Border = 0
                    Else
                        .Border = Common.TranslateBorders(Style.Border)
                        If Not Style.Border.Top.IsEmpty Then
                            .BorderWidthTop = Style.Border.Top.Width.ToPoints
                            Common.SetColor(.BorderColorTop, Style.Border.Top.Color)
                        End If
                        If Not Style.Border.Right.IsEmpty Then
                            .BorderWidthRight = Style.Border.Right.Width.ToPoints
                            Common.SetColor(.BorderColorRight, Style.Border.Right.Color)
                        End If
                        If Not Style.Border.Bottom.IsEmpty Then
                            .BorderWidthBottom = Style.Border.Bottom.Width.ToPoints
                            Common.SetColor(.BorderColorBottom, Style.Border.Bottom.Color)
                        End If
                        If Not Style.Border.Left.IsEmpty Then
                            .BorderWidthLeft = Style.Border.Left.Width.ToPoints
                            Common.SetColor(.BorderColorLeft, Style.Border.Left.Color)
                        End If
                    End If
                    If ColumnSpan > 0 Then .Colspan = ColumnSpan
                    If RowSpan > 0 Then .Rowspan = RowSpan
                    Cell.NoWrap = Not Wrap
                    Cell.UseDescender = True
                    Common.SetHorizontalAlignment(.HorizontalAlignment, Style.HorizontalAlign)
                    Common.SetVerticalAlignment(.VerticalAlignment, Style.VerticalAlign)
                    Common.SetColor(.BackgroundColor, Style.BackColor)
                    Dim Font As iTextSharp.text.Font = Common.CreatePDFFont(Style)
                    If Font Is Nothing Then
                        Paragraph = New iTextSharp.text.Paragraph(Text)
                    Else
                        Paragraph = New iTextSharp.text.Paragraph(Text, Font)
                    End If
                    If Not Style.Padding.Left.IsEmpty Then Paragraph.IndentationLeft = Style.Padding.Left.ToPoints()
                    If Not Style.Padding.Right.IsEmpty Then Paragraph.IndentationRight = Style.Padding.Right.ToPoints()
                    If Not Style.Padding.Top.IsEmpty Then Paragraph.SpacingBefore = Style.Padding.Top.ToPoints()
                    If Not Style.Padding.Bottom.IsEmpty Then
                        Paragraph.SpacingAfter = Style.Padding.Bottom.ToPoints()
                    End If
                    .AddElement(Paragraph)
                End With
                Table.AddCell(Cell)
            End If
        End Sub
#End Region

    End Class
End Namespace
