Namespace WESNet.PDF
    Public Class TableCellCollection
        Implements ICollection, IEnumerable
        Private _TableCells As ArrayList
        Private _Owner As WESNet.PDF.TableRow

        Public ReadOnly Property Item(ByVal index As Integer) As WESNet.PDF.TableCell
            Get
                Return CType(_TableCells(index), WESNet.PDF.TableCell)
            End Get
        End Property

        Public Sub Add(ByVal Cell As WESNet.PDF.TableCell)
            AddAt(-1, Cell)
        End Sub

        Public Sub AddAt(ByVal index As Integer, ByVal Cell As WESNet.PDF.TableCell)
            Cell.SetOwner(Owner)
            If index = -1 Then
                _TableCells.Add(Cell)
            Else
                _TableCells.Insert(index, Cell)
            End If
        End Sub

        Public Sub Clear()
            _TableCells.Clear()
        End Sub

        Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements ICollection.CopyTo
            Dim enumerator1 As IEnumerator = Me.GetEnumerator
            Do While enumerator1.MoveNext
                array.SetValue(enumerator1.Current, index)
                index += 1
            Loop
        End Sub

        Public ReadOnly Property Count() As Integer Implements ICollection.Count
            Get
                Return _TableCells.Count
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return _TableCells.GetEnumerator
        End Function

        Public Function IndexOf(ByVal Cell As WESNet.PDF.TableCell) As Integer
            If Not Cell Is Nothing Then
                Return _TableCells.IndexOf(Cell)
            Else
                Return -1
            End If
        End Function

        Public Sub Remove(ByVal Cell As WESNet.PDF.TableCell)
            Dim i As Integer = IndexOf(Cell)
            If i >= 0 Then RemoveAt(i)
        End Sub

        Public Sub RemoveAt(ByVal index As Integer)
            If index < 0 Or index >= Count() Then
                Throw New ArgumentOutOfRangeException("Column index out of range")
            Else
                _TableCells.RemoveAt(index)
            End If
        End Sub

        Public Sub New(ByVal Owner As WESNet.PDF.TableRow)
            _Owner = Owner
            _TableCells = New ArrayList
        End Sub

        Public Sub New(ByVal Owner As WESNet.PDF.TableRow, ByVal Cells As ArrayList)
            _Owner = Owner
            _TableCells = Cells
        End Sub

        Public ReadOnly Property IsReadOnly() As Boolean
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
            Get
                Return False
            End Get
        End Property

        Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
            Get
                Return Me
            End Get
        End Property

        Public ReadOnly Property Owner() As WESNet.PDF.TableRow
            Get
                Return _Owner
            End Get
        End Property

    End Class
End Namespace
