
Namespace WESNet.PDF

    Public Class Token

#Region "Private Members"
        Private _Name As String
        Private _DataType As Type
        Private _Value As Object
        Private _Instances As ArrayList
#End Region

#Region "Public Properties"
        Public ReadOnly Property Name() As String
            Get
                Return _Name
            End Get
        End Property

        Public ReadOnly Property DataType() As Type
            Get
                Return _DataType
            End Get
        End Property

        Public Property Value() As Object
            Get
                Return _Value
            End Get
            Set(ByVal Value As Object)
                _Value = Value
            End Set
        End Property

        Public ReadOnly Property Instances() As ArrayList
            Get
                If _Instances Is Nothing Then
                    _Instances = New ArrayList(5)
                End If
                Return _Instances
            End Get
        End Property

        Public ReadOnly Property LastInstance() As TokenInstance
            Get
                Dim Count As Integer = Instances.Count
                If Count = 0 Then
                    Return Nothing
                Else
                    Return CType(Instances(Count - 1), TokenInstance)
                End If
            End Get
        End Property
#End Region

#Region "Constructors"
        Public Sub New(ByVal Name As String, ByVal DataType As Type)
            _Name = Name
            _DataType = DataType
        End Sub
#End Region

#Region "Public Methods"

        Public Overloads Function AddInstance() As Token
            Instances.Add(New TokenInstance(Me, Instances.Count, Nothing, Nothing))
            Return Me
        End Function

        Public Overloads Function AddInstance(ByVal Font As iTextSharp.text.Font) As Token
            Instances.Add(New TokenInstance(Me, Instances.Count, Font, Nothing))
            Return Me
        End Function
        Public Overloads Function AddInstance(ByVal Font As iTextSharp.text.Font, ByVal DataFormatString As String) As Token
            Instances.Add(New TokenInstance(Me, Instances.Count, Font, DataFormatString))
            Return Me
        End Function

        Public Sub SetValue(ByVal Value As Object)
            Dim Instance As TokenInstance
            Dim Text As String
            Dim CB As iTextSharp.text.pdf.PdfContentByte
            Dim BaseFont As iTextSharp.text.pdf.BaseFont
            Dim FontSize As Single
            Dim FontColor As iTextSharp.text.Color
            If Value.GetType.Equals(DataType) Then
                _Value = Value
                For Each Instance In Instances
                    If Instance.DataFormatString Is Nothing Then
                        Text = Value.ToString()
                    Else
                        Text = String.Format(Instance.DataFormatString, Value)
                    End If
                    If Not Text Is Nothing AndAlso Text.Length > 0 Then
                        If Instance.Template Is Nothing Then
                            Exit Sub
                            Throw New iTextSharp.text.DocumentException("Template for token " & Instance.Name & " was not created")
                        Else
                            CB = Instance.DirectContent
                            If CB Is Nothing Then
                                Throw New iTextSharp.text.DocumentException("PDFContentByte for token " & Instance.Name & " was not registered")
                            Else
                                If Instance.Font Is Nothing Then
                                    BaseFont = iTextSharp.text.pdf.BaseFont.CreateFont(iTextSharp.text.pdf.BaseFont.HELVETICA, iTextSharp.text.pdf.BaseFont.CP1252, False)
                                    FontSize = Instance.Template.Height * 0.66!
                                    FontColor = iTextSharp.text.Color.BLACK
                                Else
                                    BaseFont = Instance.Font.BaseFont
                                    FontSize = Instance.Font.Size
                                    FontColor = Instance.Font.Color
                                End If
                                With Instance.Template
                                    .BeginText()
                                    .SetFontAndSize(BaseFont, FontSize)
                                    .SetRGBColorFill(FontColor.R, FontColor.G, FontColor.B)
                                    .SetTextMatrix(0.0!, -Instance.BaselineOffset)
                                    .ShowText(Text)
                                    .EndText()
                                End With
                            End If
                        End If
                    End If
                Next
            Else
                Throw New InvalidCastException("Value for token " & Name & " must be of type " & DataType.GetType.ToString)
            End If
        End Sub
#End Region

    End Class


    Public Class TokenInstance
        Private _Parent As Token
        Private _InstanceNumber As Integer
        Private _DataFormatString As String
        Private _Font As iTextSharp.text.Font
        Private _Template As iTextSharp.text.pdf.PdfTemplate
        Private _BaselineOffset As Single
        Private _DirectContent As iTextSharp.text.pdf.PdfContentByte
        Private _X As Single
        Private _Y As Single

        Public ReadOnly Property Parent() As Token
            Get
                Return _Parent
            End Get
        End Property

        Public ReadOnly Property InstanceNumber() As Integer
            Get
                Return _InstanceNumber
            End Get
        End Property

        Public ReadOnly Property Name() As String
            Get
                Return Parent.Name & ":" & InstanceNumber.ToString
            End Get
        End Property

        Public ReadOnly Property DataFormatString() As String
            Get
                Return _DataFormatString
            End Get
        End Property

        Public ReadOnly Property Font() As iTextSharp.text.Font
            Get
                Return _Font
            End Get
        End Property

        Public ReadOnly Property Template() As iTextSharp.text.pdf.PdfTemplate
            Get
                Return _Template
            End Get
        End Property

        Public ReadOnly Property X() As Single
            Get
                Return _X
            End Get
        End Property

        Public ReadOnly Property Y() As Single
            Get
                Return _Y
            End Get
        End Property

        Public ReadOnly Property BaselineOffset() As Single
            Get
                Return _BaselineOffset
            End Get
        End Property

        Public ReadOnly Property DirectContent() As iTextSharp.text.pdf.PdfContentByte
            Get
                Return _DirectContent
            End Get
        End Property

#Region "Constructors"
        Public Sub New(ByVal Parent As Token, ByVal InstanceNumber As Integer, ByVal Font As iTextSharp.text.Font, ByVal DataFormatString As String)
            _Parent = Parent
            _InstanceNumber = InstanceNumber
            _DataFormatString = DataFormatString
            _Font = Font
        End Sub

        Public Sub New(ByVal Parent As Token, ByVal InstanceNumber As Integer, ByVal Font As iTextSharp.text.Font)
            Me.New(Parent, InstanceNumber, Font, Nothing)
        End Sub

        Public Sub New(ByVal Parent As Token, ByVal InstanceNumber As Integer)
            Me.New(Parent, InstanceNumber, Nothing, Nothing)
        End Sub
#End Region

#Region "Public Methods"

        'Inserts a token placeholder into the PDF document as a iTextSharp.text.pdf.pdfTemplate

        Public Sub InsertTemplate(ByVal DirectContent As iTextSharp.text.pdf.PdfContentByte, _
                                       ByVal Rect As iTextSharp.text.Rectangle, ByVal BaselineOffset As Single)

            Dim Basefont As iTextSharp.text.pdf.BaseFont
            Dim FontSize As Single
            _DirectContent = DirectContent
            If BaselineOffset = 0.0! Then
                If Font Is Nothing OrElse Font.BaseFont Is Nothing Then
                    Basefont = iTextSharp.text.pdf.BaseFont.CreateFont(iTextSharp.text.pdf.BaseFont.HELVETICA, iTextSharp.text.pdf.BaseFont.CP1252, False)
                    FontSize = Rect.Height * 0.66!
                Else
                    Basefont = Font.BaseFont
                    FontSize = Font.Size
                End If
                _BaselineOffset = Basefont.GetDescentPoint("j", FontSize) 'Value will be a negative
            End If
            _Template = _DirectContent.CreateTemplate(Rect.Width, Rect.Height - _BaselineOffset)
            _X = Rect.Left + 10.0!
            _Y = Rect.Bottom + _BaselineOffset - 15.0!
            _DirectContent.AddTemplate(_Template, Rect.Left, Rect.Bottom + _BaselineOffset)
        End Sub

#End Region

    End Class
End Namespace
