Imports System.ComponentModel
Imports System.Globalization
Imports System.Text

Namespace WESNet

    'Defines a structure much like the System.Web.Unit which encapsulates both value and unit
    'type for various measurements and provides conversion between values of different unit types

    <TypeConverter(GetType(WESNet.UnitConverter))> _
    Public Structure Unit

#Region "Private Members"
        Private _Type As System.Web.UI.WebControls.UnitType
        Private _Value As Single
#End Region

#Region "Private Constants"
        Private Const MaxPoints As Single = 14400.0
        Private Const PointsPerCm As Single = PointsPerInch / 2.54
        Private Const PointsPerMm As Single = PointsPerInch / 2540.0
        Private Const PointsPerInch As Single = 72.0
        Private Const PointsPerPica As Single = 12.0
        Private Const PixelsPerInch As Single = 96.0
        Private Const PointsPerPixel As Single = PointsPerInch / PixelsPerInch
        Private Const PixelsPerPoint As Single = PixelsPerInch / PointsPerInch
        Private Const MaxPixels As Single = MaxPoints * PixelsPerPoint
        Private Const MaxInches As Single = MaxPoints / PointsPerInch
#End Region

#Region "Public Member"
        Public Shared ReadOnly Empty As Unit
#End Region

#Region "Public Properties"

        Public ReadOnly Property IsEmpty() As Boolean
            Get
                Return _Type = 0
            End Get
        End Property

        Public ReadOnly Property Type() As System.Web.UI.WebControls.UnitType
            Get
                If _Type = 0 Then
                    Return Web.UI.WebControls.UnitType.Point
                Else
                    Return _Type
                End If
            End Get
        End Property

        Public ReadOnly Property Value() As Single
            Get
                Return _Value
            End Get
        End Property
#End Region

#Region "Private Constructor"
        Public Sub New(ByVal Value As Single, ByVal Type As System.Web.UI.WebControls.UnitType)
            _Type = Type
            _Value = Value
        End Sub
#End Region

#Region "Public Methods"

        Public Shadows Function Equals(ByVal obj As Object) As Boolean
            If obj Is Nothing Then
                Return False
            Else
                Dim UnitObj As Unit = CType(obj, Unit)
                Return Me._Type = UnitObj._Type And Me._Value = UnitObj._Value
            End If
        End Function

        Public Overloads Function ToString(ByVal culture As CultureInfo) As String
            If IsEmpty Then Return String.Empty
            Select Case Type
                Case Web.UI.WebControls.UnitType.Cm, Web.UI.WebControls.UnitType.Em, Web.UI.WebControls.UnitType.Ex, _
                     Web.UI.WebControls.UnitType.Inch, Web.UI.WebControls.UnitType.Mm, Web.UI.WebControls.UnitType.Percentage
                    Return Value.ToString(culture) & GetStringFromType(Type)
                Case Web.UI.WebControls.UnitType.Pica, Web.UI.WebControls.UnitType.Pixel, Web.UI.WebControls.UnitType.Point
                    Return Value.ToString(culture) & GetStringFromType(Type)
            End Select
        End Function

        Public Overloads Overrides Function ToString() As String
            If IsEmpty Then Return String.Empty
            Select Case Type
                Case Web.UI.WebControls.UnitType.Cm, Web.UI.WebControls.UnitType.Em, Web.UI.WebControls.UnitType.Ex, _
                     Web.UI.WebControls.UnitType.Inch, Web.UI.WebControls.UnitType.Mm, Web.UI.WebControls.UnitType.Percentage
                    Return Value.ToString("####0.0") & GetStringFromType(Type)
                Case Web.UI.WebControls.UnitType.Pica, Web.UI.WebControls.UnitType.Pixel, Web.UI.WebControls.UnitType.Point
                    Return Value.ToString("####0") & GetStringFromType(Type)
            End Select
        End Function

        Public Function ToPoints() As Single
            If IsEmpty OrElse Type = Web.UI.WebControls.UnitType.Point OrElse Value = 0.0 Then
                Return Value
            Else
                Select Case Type
                    Case Web.UI.WebControls.UnitType.Cm
                        Return Value * PointsPerCm
                    Case Web.UI.WebControls.UnitType.Em
                        Throw New NotSupportedException("Cannot convert unit from ems to points")
                    Case Web.UI.WebControls.UnitType.Ex
                        Throw New NotSupportedException("Cannot convert unit from exs to points")
                    Case Web.UI.WebControls.UnitType.Inch
                        Return Value * PointsPerInch
                    Case Web.UI.WebControls.UnitType.Mm
                        Return Value * PointsPerMm
                    Case Web.UI.WebControls.UnitType.Percentage
                        Throw New NotSupportedException("Cannot convert unit from percent to points")
                    Case Web.UI.WebControls.UnitType.Pica
                        Return Value * PointsPerPica
                    Case Web.UI.WebControls.UnitType.Pixel
                        Return Value * PixelsPerPoint
                End Select
            End If
        End Function

        Public Sub SetPoints(ByRef Value As Single)
            If Not IsEmpty Then
                Value = ToPoints()
            End If
        End Sub

#End Region

#Region "Public Shared Methods"

        Public Overloads Shared Function Parse(ByVal s As String, ByVal culture As CultureInfo) As Unit
            If s Is Nothing Then Return Unit.Empty
            If culture Is Nothing Then
                culture = CultureInfo.CurrentCulture
            End If
            s = s.Trim.ToLower(CultureInfo.InvariantCulture)
            If s.Length = 0 Then
                Return New Unit(0, Web.UI.WebControls.UnitType.Point)
            Else
                Dim rgx As New RegularExpressions.Regex("^(\d+[\.\,]{0,1}\d*)\s*(cm|em|ex|in|pc|pt|px|%){0,1}")
                Dim m As RegularExpressions.Match = rgx.Match(s.ToLower)
                If m.Success Then
                    Dim n As Single
                    Dim t As Web.UI.WebControls.UnitType
                    If m.Groups.Count = 3 Then
                        t = GetTypeFromString(m.Groups(2).Value)
                    Else
                        t = Web.UI.WebControls.UnitType.Point
                    End If
                    Try
                        If (t = Web.UI.WebControls.UnitType.Pixel) Then
                            Dim converter1 As New Int32Converter
                            n = CType(converter1.ConvertFromString(Nothing, culture, m.Groups(1).Value), Integer)
                        Else
                            Dim converter2 As New SingleConverter
                            n = CType(converter2.ConvertFromString(Nothing, culture, m.Groups(1).Value), Single)
                        End If

                    Catch ex As Exception
                        Throw New FormatException("Invalid unit numeric value (" & m.Groups(1).Value & ")")
                    End Try
                    Return New Unit(n, t)
                Else
                    Return Unit.Empty
                End If
            End If
        End Function

        Public Overloads Shared Function Parse(ByVal s As String) As Unit
            Return Parse(s, CultureInfo.CurrentCulture)
        End Function

        Public Shared Function Percentage(ByVal Value As Single) As Unit
            If Value < 0 Or Value > 100.0 Then
                Throw New ArgumentOutOfRangeException("Unit percentage values must range from 0.00 to 100.00")
            End If
            Return New Unit(Value, Web.UI.WebControls.UnitType.Percentage)
        End Function

        Public Shared Function Pixel(ByVal Value As Single) As Unit

            If Value < -MaxPixels Or Value > MaxPixels Then
                Throw New ArgumentOutOfRangeException("Unit pixel values must range from " & (-MaxPixels).ToString & " to " & MaxPixels.ToString)
            End If
            Return New Unit(Value, Web.UI.WebControls.UnitType.Pixel)
        End Function

        Public Shared Function Inch(ByVal Value As Single) As Unit
            If Value < -MaxInches Or Value > MaxInches Then
                Throw New ArgumentOutOfRangeException("Unit inch values must range from " & (-MaxInches).ToString & " to " & MaxInches.ToString)
            End If
            Return New Unit(Value, Web.UI.WebControls.UnitType.Inch)
        End Function

        Public Shared Function Point(ByVal Value As Single) As Unit
            If Value < -MaxPoints Or Value > MaxPoints Then
                Throw New ArgumentOutOfRangeException("Unit point values must range from " & (-MaxPoints).ToString & " to " & MaxPoints.ToString)
            End If
            Return New Unit(Value, Web.UI.WebControls.UnitType.Point)
        End Function

#End Region

#Region "Private Methods"
        Private Shared Function GetTypeFromString(ByVal s As String) As System.Web.UI.WebControls.UnitType
            If s Is Nothing OrElse s.Length = 0 Then Return Web.UI.WebControls.UnitType.Point
            Select Case s.ToLower
                Case "cm"
                    Return Web.UI.WebControls.UnitType.Cm
                Case "em"
                    Return Web.UI.WebControls.UnitType.Em
                Case "ex"
                    Return Web.UI.WebControls.UnitType.Ex
                Case "in"
                    Return Web.UI.WebControls.UnitType.Inch
                Case "mm"
                    Return Web.UI.WebControls.UnitType.Mm
                Case "%"
                    Return Web.UI.WebControls.UnitType.Percentage
                Case "pc"
                    Return Web.UI.WebControls.UnitType.Pica
                Case "px"
                    Return Web.UI.WebControls.UnitType.Pixel
                Case "pt"
                    Return Web.UI.WebControls.UnitType.Point
                Case Else
                    Throw New ArgumentException("Invalid units specifier in unit string")
            End Select
        End Function

        Private Shared Function GetStringFromType(ByVal Type As System.Web.UI.WebControls.UnitType) As String
            Select Case Type
                Case Web.UI.WebControls.UnitType.Point
                    Return "pt"
                Case Web.UI.WebControls.UnitType.Pixel
                    Return "px"
                Case Web.UI.WebControls.UnitType.Inch
                    Return "in"
                Case Web.UI.WebControls.UnitType.Percentage
                    Return "%"
                Case Web.UI.WebControls.UnitType.Mm
                    Return "mm"
                Case Web.UI.WebControls.UnitType.Cm
                    Return "cm"
                Case Web.UI.WebControls.UnitType.Em
                    Return "em"
                Case Web.UI.WebControls.UnitType.Ex
                    Return "ex"
                Case Web.UI.WebControls.UnitType.Pica
                    Return "pc"
                Case Else
                    Return String.Empty
            End Select
        End Function
#End Region

    End Structure
End Namespace
